""" Main module
"""

import os
import re
import json
import pprint
import datetime
import functools
import traceback
from hashlib import md5
from concurrent.futures import ThreadPoolExecutor

import tornado.web
import tornado.websocket
from tornado import gen
from tornado.web import HTTPError
from tornado.ioloop import IOLoop

from DIRAC import gLogger, gConfig, S_OK, S_ERROR
from DIRAC.Core.Utilities.JEncode import DATETIME_DEFAULT_FORMAT
from DIRAC.Core.Utilities.Decorators import deprecated
from DIRAC.Core.DISET.AuthManager import AuthManager
from DIRAC.Core.DISET.ThreadConfig import ThreadConfig
from DIRAC.Core.Tornado.Server.TornadoREST import (
    TornadoREST,
    authorization,
    TornadoResponse,
)  # pylint: disable=no-name-in-module
from DIRAC.FrameworkSystem.private.authorization.utils.Tokens import OAuth2Token

from WebAppDIRAC.Lib import Conf
from WebAppDIRAC.Lib.SessionData import SessionData


global gThreadPool
gThreadPool = ThreadPoolExecutor(100)
sLog = gLogger.getSubLogger(__name__)


class UserCredentials:
    """With this class you can make RPC calls using user credantials

    Usage:

        def post_job(self):
            with UserCredentials(**self.credDict):
                # Do stuff

    """

    __disetConfig = ThreadConfig()

    def __init__(self, **kwargs):
        self.dn = kwargs["DN"]
        self.group = kwargs["group"]
        self.setup = kwargs["setup"]

    def __enter__(self):
        self.__disetConfig.load((self.dn, self.group, self.setup))
        return self.__disetConfig

    def __exit__(self, type, value, traceback):
        self.__disetConfig.reset()


class FileResponse(TornadoResponse):
    """This class provide logic for CSV and PNG formats.

    Usage example::

      def web_myMethod(self):
        # Generate CSV data
        ...
        return FileResponse(data, 'filename', 'csv')
    """

    def __init__(self, payload, fileName: str, ext: str = "", cache: bool = True):
        """C'or

        :param payload: response body
        :param fileName: CSV name
        :param ext: file type
        :param cache: use cache
        """
        name, _ext = os.path.splitext(fileName)
        self.ext = (ext or _ext).lower()
        # Generate file name
        self.fileHash = md5(name.encode()).hexdigest()  # MD5 take a bytes
        self.cache = cache
        super().__init__(payload, 200)

    def _runActions(self, reqObj):
        """Calling methods in the order of their registration

        :param reqObj: RequestHandler instance
        """
        # Set content type
        if self.ext == "csv":
            reqObj.set_header("Content-type", "text/csv")
        elif self.ext == "png":
            reqObj.set_header("Content-Transfer-Encoding", "Binary")
            reqObj.set_header("Content-type", "image/png")
        else:
            reqObj.set_header("Content-type", "text/plain")

        reqObj.set_header("Content-Disposition", f'attachment; filename="{self.fileHash}.{self.ext}"')
        reqObj.set_header("Content-Length", len(self.payload))

        if not self.cache:
            # Disable cache
            reqObj.set_header("Cache-Control", "no-cache, no-store, must-revalidate, max-age=0")
            reqObj.set_header("Pragma", "no-cache")
            reqObj.set_header(
                "Expires",
                (datetime.datetime.utcnow() - datetime.timedelta(minutes=-10)).strftime("%d %b %Y %H:%M:%S GMT"),
            )

        super()._runActions(reqObj)


class WErr(HTTPError):
    def __init__(self, code, msg="", **kwargs):
        super().__init__(code, str(msg) or None)
        for k in kwargs:
            setattr(self, k, kwargs[k])
        self.msg = msg
        self.kwargs = kwargs

    @classmethod
    def fromSERROR(cls, result):
        """Prevent major problem with % in the message"""
        return cls(500, result["Message"].replace("%", ""))


def asyncWithCallback(method):
    return tornado.web.asynchronous(method)


def asyncGen(method):
    return gen.coroutine(method)


def defaultEncoder(data):
    """Encode

      - datetime to ISO format string
      - set to list

    :param data: value to encode

    :return: encoded value
    """
    if isinstance(data, (datetime.date, datetime.datetime)):
        return data.strftime(DATETIME_DEFAULT_FORMAT)
    if isinstance(data, (set)):
        return list(data)
    raise TypeError("Object of type {} is not JSON serializable".format(data.__class__.__name__))


class _WebHandler(TornadoREST):
    __session = None
    __sessionData = None
    __disetConfig = ThreadConfig()

    DEFAULT_AUTHENTICATION = ["SSL", "SESSION", "VISITOR"]
    # Auth requirements DEFAULT_AUTHORIZATION
    DEFAULT_AUTHORIZATION = None
    # Base URL prefix
    BASE_URL = None
    # Location of the handler in the URL
    DEFAULT_LOCATION = ""
    # RE to extract group and setup
    PATH_RE = None
    # Prefix of methods names
    METHOD_PREFIX = "web_"

    SUPPORTED_METHODS = (
        "POST",
        "GET",
    )

    # for backward compatibility
    LOCATION = ""
    AUTH_PROPS = None

    # pylint: disable=no-member
    @classmethod
    def _pre_initialize(cls):
        # For backward compatibility
        cls.LOCATION = cls.LOCATION or cls.DEFAULT_LOCATION
        cls.AUTH_PROPS = cls.AUTH_PROPS or cls.DEFAULT_AUTHORIZATION

        cls.DEFAULT_LOCATION = cls.DEFAULT_LOCATION or cls.LOCATION
        cls.DEFAULT_AUTHORIZATION = cls.DEFAULT_AUTHORIZATION or cls.AUTH_PROPS

        # Get tornado URLs
        urls = super()._pre_initialize()
        # Define base path regex to know setup/group
        cls.PATH_RE = re.compile(f"{cls.BASE_URL}(.*)")
        return urls

    def finish(self, data=None, *args, **kwargs):
        """Finishes this response, ending the HTTP request. More detailes:
        https://www.tornadoweb.org/en/stable/_modules/tornado/web.html#RequestHandler.finish
        """
        if data and isinstance(data, dict):
            self.set_header("Content-Type", "application/json")
            data = json.dumps(data, default=defaultEncoder)
        return super().finish(data, *args, **kwargs)

    @classmethod
    def _getCSAuthorizarionSection(cls, handler):
        """Search endpoint auth section.

        :param str handler: API name, see :py:meth:`_getFullComponentName`

        :return: str
        """
        return Conf.getAuthSectionForHandler(handler)

    def _getMethodArgs(self, args: tuple, kwargs: dict):
        """Decode args.

        :return: tuple(list, dict)
        """
        return super()._getMethodArgs(args=args[3:], kwargs=kwargs)

    async def prepare(self):
        """Prepare the request. It reads certificates and check authorizations.
        We make the assumption that there is always going to be a ``method`` argument
        regardless of the HTTP method used
        """

        # Parse request URI
        match = self.PATH_RE.match(self.request.path)
        groups = match.groups()
        self.__setup = groups[0] or Conf.setup()
        self.__group = groups[1]

        # Reset DISET settings
        self.__disetConfig.reset()
        self.__disetConfig.setDecorator(self.__disetBlockDecor)
        self.__disetDump = self.__disetConfig.dump()

        try:
            await super().prepare()
        except HTTPError as e:
            raise WErr(e.status_code, e.log_message)

        # Configure DISET with user creds
        if self.getUserDN():
            self.__disetConfig.setDN(self.getUserDN())
        if self.getUserGroup():  # pylint: disable=no-value-for-parameter
            self.__disetConfig.setGroup(self.getUserGroup())  # pylint: disable=no-value-for-parameter
        self.__disetConfig.setSetup(self.__setup)
        self.__disetDump = self.__disetConfig.dump()

        self.__forceRefreshCS()

    def __disetBlockDecor(self, func):
        def wrapper(*args, **kwargs):
            raise RuntimeError("All DISET calls must be made from inside a Threaded Task!")

        return wrapper

    def __forceRefreshCS(self):
        """Force refresh configuration from master configuration server"""
        if self.request.headers.get("X-RefreshConfiguration") == "True":
            self.log.debug("Initialize force refresh..")
            if not AuthManager("").authQuery("", dict(self.credDict), "CSAdministrator"):
                raise WErr(401, "Cannot initialize force refresh, request not authenticated")
            result = gConfig.forceRefresh()
            if not result["OK"]:
                raise WErr(501, result["Message"])

    def _gatherPeerCredentials(self):
        """
        Load client certchain in DIRAC and extract informations.

        The dictionary returned is designed to work with the AuthManager,
        already written for DISET and re-used for HTTPS.

        :returns: a dict containing the return of :py:meth:`DIRAC.Core.Security.X509Chain.X509Chain.getCredentials`
                  (not a DIRAC structure !)
        """
        # Authorization type
        self.__authGrant = ["VISITOR"]
        if self.request.protocol == "https":
            # First of all we try to authZ with what is specified in cookies, and if attempt is unsuccessful authZ as visitor
            self.__authGrant.insert(0, self.get_cookie("authGrant", "SSL").replace("Certificate", "SSL"))

        credDict = super()._gatherPeerCredentials(grants=self.__authGrant)

        # Add a group if it present in the request path
        if credDict and self.__group:
            credDict["validGroup"] = False
            credDict["group"] = self.__group

        return credDict

    def _authzSESSION(self):
        """Fill credentionals from session

        :return: S_OK(dict)
        """
        credDict = {}

        # Session
        sessionID = self.get_secure_cookie("session_id")

        if not sessionID:
            self.clear_cookie("authGrant")
            return S_OK(credDict)

        # Each session depends on the tokens
        try:
            sLog.debug("Load session tokens..")
            token = OAuth2Token(sessionID.decode())
            sLog.debug("Found session tokens:\n", pprint.pformat(token))
            try:
                return self._authzJWT(token["access_token"])
            except Exception as e:
                sLog.debug("Cannot check access token %s, try to fetch.." % repr(e))
                # Try to refresh access_token and refresh_token
                result = self._idps.getIdProvider("DIRACWeb")
                if not result["OK"]:
                    return result
                cli = result["Value"]
                token = cli.refreshToken(token["refresh_token"])
                # store it to the secure cookie
                self.set_secure_cookie("session_id", json.dumps(token), secure=True, httponly=True)
                return self._authzJWT(token["access_token"])

        except Exception as e:
            sLog.debug(repr(e))
            # if attempt is unsuccessful expire session
            self.clear_cookie("session_id")
            self.set_cookie("session_id", "expired")
            self.set_cookie("authGrant", "Visitor")
            return S_ERROR(repr(e))

    @property
    def log(self):
        return sLog

    @classmethod
    def getLog(cls):
        return sLog

    def getCurrentSession(self):
        return self.__session

    def getUserSetup(self):
        return self.__setup

    def getSessionData(self):
        if not self.__sessionData:
            self.__sessionData = SessionData(self.credDict, self.__setup)
        return self.__sessionData.getData()

    def getAppSettings(self, app=None):
        return Conf.getAppSettings(app or self.__class__.__name__.replace("Handler", "")).get("Value") or {}

    def write_error(self, status_code, **kwargs):
        self.set_status(status_code)
        cType = "text/plain"
        data = self._reason
        if "exc_info" in kwargs:
            ex = kwargs["exc_info"][1]
            trace = traceback.format_exception(*kwargs["exc_info"])
            if not isinstance(ex, WErr):
                data += "\n".join(trace)
            else:
                if self.settings.get("debug"):
                    self.log.error("Request ended in error:\n  %s" % "\n  ".join(trace))
                data = ex.msg
                if isinstance(data, dict):
                    cType = "application/json"
                    data = json.dumps(data)
        self.set_header("Content-Type", cType)
        self.finish(data)

    @deprecated("Should be deprecated for v5+, use FileResponse class instead")
    def finishWithImage(self, data, plotImageFile, disableCaching=False):
        # Set headers
        self.set_header("Content-Type", "image/png")
        self.set_header(
            "Content-Disposition", 'attachment; filename="%s.png"' % md5(plotImageFile.encode()).hexdigest()
        )
        self.set_header("Content-Length", len(data))
        self.set_header("Content-Transfer-Encoding", "Binary")
        if disableCaching:
            self.set_header("Cache-Control", "no-cache, no-store, must-revalidate, max-age=0")
            self.set_header("Pragma", "no-cache")
            self.set_header(
                "Expires",
                (datetime.datetime.utcnow() - datetime.timedelta(minutes=-10)).strftime("%d %b %Y %H:%M:%S GMT"),
            )
        # Return the data
        self.finish(data)


class WebHandler(_WebHandler):
    """Old WebHandler"""

    @classmethod
    def _pre_initialize(cls):
        # Get tornado URLs
        urls = super()._pre_initialize()
        # Add a pattern that points to the target method.
        # Note that there are handlers with an index method.
        # It responds to the request without specifying a method.
        # The special characters "*" helps to take into account such a case,
        # see https://docs.python.org/3/library/re.html#regular-expression-syntax.
        # E.g .: /DIRAC/ -> RootHandler.web_index
        cls.PATH_RE = re.compile(f"{cls.BASE_URL}({cls.LOCATION}/[A-z0-9_]*)")
        return urls

    def get(self, setup, group, *pathArgs):
        self.initializeRequest()
        return self._getMethod()(*pathArgs)

    def post(self, *args, **kwargs):
        return self.get(*args, **kwargs)

    def threadTask(self, method, *args, **kwargs):
        def threadJob(*targs, **tkwargs):
            args = targs[0]
            disetConf = targs[1]
            self.__disetConfig.reset()
            self.__disetConfig.load(disetConf)
            return method(*args, **tkwargs)

        targs = (args, self.__disetDump)
        return IOLoop.current().run_in_executor(gThreadPool, functools.partial(threadJob, *targs, **kwargs))


class WebSocketHandler(tornado.websocket.WebSocketHandler, WebHandler):
    def __init__(self, *args, **kwargs):
        WebHandler.__init__(self, *args, **kwargs)
        tornado.websocket.WebSocketHandler.__init__(self, *args, **kwargs)

    @classmethod
    def _pre_initialize(cls):
        # For backward compatibility
        cls.LOCATION = cls.LOCATION or cls.DEFAULT_LOCATION
        cls.AUTH_PROPS = cls.AUTH_PROPS or cls.DEFAULT_AUTHORIZATION

        cls.DEFAULT_LOCATION = cls.DEFAULT_LOCATION or cls.LOCATION
        cls.DEFAULT_AUTHORIZATION = cls.DEFAULT_AUTHORIZATION or cls.AUTH_PROPS

        # Define base path regex to know setup/group
        cls.PATH_RE = re.compile(url := f"{cls.BASE_URL}({cls.LOCATION})")
        sLog.verbose(f" - WebSocket {cls.LOCATION} -> {cls.__name__}")
        sLog.debug(f"  * {url}")
        return [(url, cls)]

    def open(self, *args, **kwargs):
        """Invoked when a new WebSocket is opened, read more in tornado `docs.\
        <https://www.tornadoweb.org/en/stable/websocket.html#tornado.websocket.WebSocketHandler.open>`_
        """
        return self.on_open()

    def on_open(self):
        """Developer should implement this method"""
        raise NotImplementedError('"on_open" method is not implemented')

    def _getMethod(self):
        """Get method function to call."""
        return self.on_open



import tornado.web
from urllib.parse import urlparse

from WebAppDIRAC.Lib import Conf


class CoreHandler(tornado.web.RequestHandler):
    def initialize(self, action):
        self.__action = action

    def get(self, setup, group, route):
        if self.__action == "addSlash":
            o = urlparse(self.request.uri)
            proto = self.request.protocol
            if "X-Scheme" in self.request.headers:
                proto = self.request.headers["X-Scheme"]
            nurl = "%s://%s%s/" % (proto, self.request.host, o.path)
            if o.query:
                nurl = "%s?%s" % (nurl, o.query)
            self.redirect(nurl, permanent=True)
        elif self.__action == "sendToRoot":
            dest = "/"
            rootURL = Conf.rootURL()
            if rootURL:
                dest += "%s/" % rootURL.strip("/")
            if setup and group:
                dest += "s:%s/g:%s/" % (setup, group)
            self.redirect(dest)



import os
import uuid
import tempfile
import tornado.process

from DIRAC import gConfig
from DIRAC.Core.Security import Locations, X509Chain, X509CRL

BASECS = "/WebApp"


def getCSValue(opt, defValue=None):
    """Get option from CS

    :param str opt: option
    :param defValue: default value

    :return: defValue type result
    """
    return gConfig.getValue("%s/%s" % (BASECS, opt), defValue)


def getCSSections(opt):
    """Get sections from CS

    :param str opt: option

    :return: S_OK(list)/S_ERROR()
    """
    return gConfig.getSections("%s/%s" % (BASECS, opt))


def getCSOptions(opt):
    """Get options from CS

    :param str opt: option

    :return: S_OK(list)/S_ERROR()
    """
    return gConfig.getOptions("%s/%s" % (BASECS, opt))


def getCSOptionsDict(opt):
    """Get options dictionary from CS

    :param str opt: option

    :return: S_OK(dict)/S_ERROR()
    """
    return gConfig.getOptionsDict("%s/%s" % (BASECS, opt))


def getTitle():
    """Get title

    :return: str
    """
    defVal = gConfig.getValue("/DIRAC/Configuration/Name", gConfig.getValue("/DIRAC/Setup"))
    return "%s - DIRAC" % gConfig.getValue("%s/Title" % BASECS, defVal)


def devMode():
    """Get development mode status

    :result: bool
    """
    return getCSValue("DevelopMode", False)


def rootURL():
    """Get root URL

    :return: str
    """
    return getCSValue("RootURL", "/DIRAC")


def balancer():
    """Get balancer

    :return: str
    """
    b = getCSValue("Balancer", "").lower()
    if b in ("", "none"):
        return ""
    return b


def numProcesses():
    """Get number of processes

    :return: int
    """
    return getCSValue("NumProcesses", 1)


def HTTPS():
    """Get flag of enable HTTPS

    :return: bool
    """
    if balancer():
        return False
    return getCSValue("HTTPS/Enabled", True)


def HTTPPort():
    """Get HTTP port

    :return: int
    """
    if balancer():
        default = 8000
    else:
        default = 8080
    procAdd = tornado.process.task_id() or 0
    return getCSValue("HTTP/Port", default) + procAdd


def HTTPSPort():
    """Get HTTPS port

    :return: int
    """
    return getCSValue("HTTPS/Port", 8443)


def HTTPSCert():
    """Get certificate path for HTTPS

    :return: str
    """
    cert = Locations.getHostCertificateAndKeyLocation()
    if cert:
        cert = cert[0]
    else:
        cert = "/opt/dirac/etc/grid-security/hostcert.pem"
    return getCSValue("HTTPS/Cert", cert)


def HTTPSKey():
    """Get key path for HTTPS

    :return: str
    """
    key = Locations.getHostCertificateAndKeyLocation()
    if key:
        key = key[1]
    else:
        key = "/opt/dirac/etc/grid-security/hostkey.pem"
    return getCSValue("HTTPS/Key", key)


def setup():
    """Get setup path

    :return: str
    """
    return gConfig.getValue("/DIRAC/Setup")


def cookieSecret():
    """Get cookie secret

    :return: str
    """
    # TODO: Store the secret somewhere
    return gConfig.getValue("CookieSecret", uuid.getnode())


def generateCAFile():
    """Generate a single CA file with all the PEMs

    :return: str or bool
    """
    caDir = Locations.getCAsLocation()
    for fn in (
        os.path.join(os.path.dirname(caDir), "cas.pem"),
        os.path.join(os.path.dirname(HTTPSCert()), "cas.pem"),
        False,
    ):
        if not fn:
            fn = tempfile.mkstemp(prefix="cas.", suffix=".pem")[1]
        try:
            fd = open(fn, "w")
        except IOError:
            continue
        for caFile in os.listdir(caDir):
            caFile = os.path.join(caDir, caFile)
            chain = X509Chain.X509Chain()
            result = chain.loadChainFromFile(caFile)
            if not result["OK"]:
                continue
            expired = chain.hasExpired()
            if not expired["OK"] or expired["Value"]:
                continue
            fd.write(chain.dumpAllToString()["Value"])
        fd.close()
        return fn
    return False


def generateRevokedCertsFile():
    """Generate a single CA file with all the PEMs

    :return: str or bool
    """
    caDir = Locations.getCAsLocation()
    for fn in (
        os.path.join(os.path.dirname(caDir), "allRevokedCerts.pem"),
        os.path.join(os.path.dirname(HTTPSCert()), "allRevokedCerts.pem"),
        False,
    ):
        if not fn:
            fn = tempfile.mkstemp(prefix="allRevokedCerts", suffix=".pem")[1]
        try:
            fd = open(fn, "w")
        except IOError:
            continue
        for caFile in os.listdir(caDir):
            caFile = os.path.join(caDir, caFile)
            chain = X509CRL.X509CRL()
            result = chain.loadCRLFromFile(caFile)
            if not result["OK"]:
                continue
            fd.write(chain.dumpAllToString()["Value"])
        fd.close()
        return fn
    return False


def getAuthSectionForHandler(route):
    """Get auth section for handler

    :return: str
    """
    return "%s/Access/%s" % (BASECS, route)


def getTheme():
    """Get theme

    :return: str
    """
    return getCSValue("Theme", "tabs")


def getIcon():
    """Get icon path

    :return: str
    """
    return getCSValue("Icon", "/static/core/img/icons/system/favicon.ico")


def SSLProtocol():
    """Get ssl protocol

    :return: str
    """
    return getCSValue("SSLProtocol", getCSValue("SSLProtcol", ""))


def getDefaultStaticDirs():
    """Get default static directories

    :return: list
    """
    defDirs = getCSValue("DefaultStaticDirs", ["defaults", "demo"])
    if defDirs == ["None"]:
        return []
    return defDirs


def getStaticDirs():
    """Get static directories

    :return: str
    """
    return list(set(getCSValue("StaticDirs", []) + getDefaultStaticDirs()))


def getLogo():
    """Get logo path

    :return: str
    """
    return getCSValue("Logo", "/static/core/img/icons/system/_logo_waiting.gif")


def getBackgroud():
    """Get background path

    :return: str
    """
    return getCSValue("BackgroundImage", "/static/core/img/wallpapers/dirac_background_6.png")


def getWelcome():
    """Get welcome

    :return: str
    """
    return getCSValue("WelcomeHTML", "")


def bugReportURL():
    """Get bug report URL

    :return: str
    """
    return getCSValue("bugReportURL", "")


def getAuthNames():
    """Get enabled id providers

    :return: S_OK(list)/S_ERROR()
    """
    return getCSSections("TypeAuths")


def getAppSettings(app):
    """Get applications options

    :param str app: application name

    :return: S_OK(dict)/S_ERROR
    """
    return gConfig.getOptionsDictRecursively("%s/%s" % (BASECS, app))



""" Handler for TransformationMonitor page
"""

import json

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.TransformationSystem.Client.TransformationClient import TransformationClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, WErr, asyncGen


class TransformationMonitorHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    def index(self):
        pass

    @asyncGen
    def web_getSelectionData(self):
        callback = {}
        group = self.getUserGroup()
        user = self.getUserName()
        if user == "Anonymous":
            callback["prod"] = [["Insufficient rights"]]
        else:
            callback = {}

            ####
            tsClient = TransformationClient()
            result = yield self.threadTask(tsClient.getDistinctAttributeValues, "Plugin", {})

            if result["OK"]:
                plugin = []
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        plugin.append([str(i)])
                else:
                    plugin.append("Nothing to display")
            else:
                plugin = "Error during RPC call"
            callback["plugin"] = plugin
            ####
            result = yield self.threadTask(tsClient.getDistinctAttributeValues, "Status", {})
            if result["OK"]:
                status = []
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        status.append([str(i)])
                else:
                    status = "Nothing to display"
            else:
                status = "Error during RPC call"
            callback["prodStatus"] = status
            ####
            result = yield self.threadTask(tsClient.getDistinctAttributeValues, "TransformationGroup", {})
            if result["OK"]:
                group = []
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        group.append([str(i)])
                else:
                    group = "Nothing to display"
            else:
                group = "Error during RPC call"
            callback["transformationGroup"] = group
            ####
            result = yield self.threadTask(tsClient.getDistinctAttributeValues, "AgentType", {})
            if result["OK"]:
                atype = []
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        atype.append([str(i)])
                else:
                    atype = "Nothing to display"
            else:
                atype = "Error during RPC call"
            callback["agentType"] = atype
            ####
            result = yield self.threadTask(tsClient.getDistinctAttributeValues, "Type", {})
            if result["OK"]:
                transType = []
                if result["Value"]:
                    for i in result["Value"]:
                        transType.append([str(i)])
                else:
                    transType = "Nothing to display"
            else:
                transType = "Error during RPC call"
            callback["productionType"] = transType
        self.finish(callback)

    @asyncGen
    def web_getTransformationData(self):
        pagestart = Time.time()
        user = self.getUserName()

        tsClient = TransformationClient(timeout=3600)

        if user == "Anonymous":
            callback = {"success": "false", "error": "You are not authorised"}
        else:
            result = self._request()

            result = yield self.threadTask(
                tsClient.getTransformationSummaryWeb, result, self.globalSort, self.pageNumber, self.numberOfJobs
            )
            if not result["OK"]:
                self.finish(json.dumps({"success": "false", "error": result["Message"]}))
                return

            result = result["Value"]

            if "TotalRecords" not in result:
                self.finish(json.dumps({"success": "false", "result": "", "error": "Data structure is corrupted"}))
                return

            if result["TotalRecords"] < 1:
                self.finish(
                    json.dumps(
                        {"success": "false", "result": "", "error": "There were no data matching your selection"}
                    )
                )
                return

            if "ParameterNames" not in result and "Records" not in result:
                self.finish(json.dumps({"success": "false", "result": "", "error": "Data structure is corrupted"}))
                return

            if len(result["ParameterNames"]) < 1:
                self.finish(json.dumps({"success": "false", "result": "", "error": "ParameterNames field is missing"}))
                return

            if len(result["Records"]) < 1:
                self.finish(json.dumps({"success": "false", "Message": "There are no data to display"}))
                return

            callback = []
            jobs = result["Records"]
            head = result["ParameterNames"]
            headLength = len(head)
            for i in jobs:
                tmp = {head[j]: i[j] for j in range(headLength)}
                callback.append(tmp)
            total = result["TotalRecords"]
            if "Extras" in result:
                gLogger.info(result["Extras"])
                extra = result["Extras"]
                timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
                callback = {"success": "true", "result": callback, "total": total, "extra": extra, "date": timestamp}
            else:
                callback = {"success": "true", "result": callback, "total": total, "date": None}

            gLogger.info("\033[0;31m PRODUCTION SUBMIT REQUEST: \033[0m %s" % (Time.time() - pagestart))
        self.finish(json.dumps(callback))

    ################################################################################
    @asyncGen
    def web_action(self):
        transid = int(self.get_argument("id"))
        if self.get_argument("data_kind") == "getLoggingInfo":
            callback = yield self.threadTask(self.__getLoggingInfo, transid)
        elif self.get_argument("data_kind") == "fileStatus":
            callback = yield self.threadTask(self.__transformationFileStatus, transid)
        elif self.get_argument("data_kind") == "fileProcessed":
            callback = yield self.threadTask(self.__fileRetry, transid, "proc")
        elif self.get_argument("data_kind") == "fileNotProcessed":
            callback = yield self.threadTask(self.__fileRetry, transid, "not")
        elif self.get_argument("data_kind") == "fileAllProcessed":
            callback = yield self.threadTask(self.__fileRetry, transid, "all")
        elif self.get_argument("data_kind") == "dataQuery":
            callback = yield self.threadTask(self.__dataQuery, transid)
        elif self.get_argument("data_kind") == "additionalParams":
            callback = yield self.threadTask(self.__additionalParams, transid)
        elif self.get_argument("data_kind") == "transformationDetail":
            callback = yield self.threadTask(self.__transformationDetail, transid)
        elif self.get_argument("data_kind") == "extend":
            callback = yield self.threadTask(self.__extendTransformation, transid)
        elif self.get_argument("data_kind") == "workflowxml":
            callback = yield self.threadTask(self.__workflowxml, transid)
        else:
            callback = {"success": "false", "error": "Action is unknown!!!"}
        self.finish(callback)

    ################################################################################
    @asyncGen
    def web_executeOperation(self):
        cmd = self.get_argument("action")
        ids = self.get_argument("ids").split(",")
        ids = [int(i) for i in ids]

        tsClient = TransformationClient()

        agentType = "Manual"
        if cmd == "clean":
            status = "Cleaning"
        elif cmd == "start":
            status = "Active"
            agentType = "Automatic"
        elif cmd == "flush":
            status = "Flush"
            agentType = "Automatic"
        elif cmd == "stop":
            status = "Stopped"
        elif cmd == "complete":
            status = "Completed"
        else:
            self.finish({"success": "false", "error": "Unknown action"})

        callback = []

        for i in ids:
            try:
                transid = int(i)

                result = yield self.threadTask(tsClient.setTransformationParameter, transid, "Status", status)

                if result["OK"]:
                    resString = "ProdID: %s set to %s successfully" % (i, cmd)
                    result = yield self.threadTask(tsClient.setTransformationParameter, transid, "AgentType", agentType)
                    if not result["OK"]:
                        resString = "ProdID: %s failed to set to %s: %s" % (i, cmd, result["Message"])
                else:
                    resString = "ProdID: %s failed due the reason: %s" % (i, result["Message"])
            except Exception:
                resString = "Unable to convert given ID %s to transformation ID" % i
            callback.append(resString)
        callback = {"success": "true", "showResult": callback}
        gLogger.info(cmd, ids)
        self.finish(callback)

    ################################################################################
    def __fileRetry(self, prodid, mode):
        tsClient = TransformationClient()
        if mode == "proc":
            res = tsClient.getTransformationFilesCount(prodid, "ErrorCount", {"Status": "Processed"})
        elif mode == "not":
            res = tsClient.getTransformationFilesCount(
                prodid, "ErrorCount", {"Status": ["Unused", "Assigned", "Failed"]}
            )
        elif mode == "all":
            res = tsClient.getTransformationFilesCount(prodid, "ErrorCount")

        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        resList = []
        total = res["Value"].pop("Total")
        if total == 0:
            return {"success": "false", "error": "No files found"}
        for status in sorted(res["Value"]):
            count = res["Value"][status]
            percent = "%.1f" % ((count * 100.0) / total)
            resList.append((status, str(count), percent))
        resList.append(("Total", total, "-"))
        gLogger.debug("#######", res)
        return {"success": "true", "result": resList}

    ################################################################################
    def __dataQuery(self, prodid):
        callback = {}
        tsClient = TransformationClient()

        # FIXME: getTransformationInputDataQuery has been replaced by getTransformationMetaQuery in DIRAC v7r0
        res = tsClient.getTransformationMetaQuery(prodid, "Input")
        if not res["OK"] and "Unknown method" in res["Message"]:
            res = tsClient.getTransformationInputDataQuery(prodid)

        gLogger.debug("-= #######", res)
        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        result = res["Value"]
        back = [[i, result[i]] for i in sorted(result)]
        return {"success": "true", "result": back}

    ################################################################################
    def __additionalParams(self, prodid):
        callback = {}
        tsClient = TransformationClient()

        res = tsClient.getAdditionalParameters(prodid)
        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        result = res["Value"]
        back = [[i, result[i]] for i in sorted(result)]
        return {"success": "true", "result": back}

    ################################################################################
    def __workflowxml(self, transid):

        tsClient = TransformationClient()
        retVal = tsClient.getTransformations({"TransformationID": transid})
        if not retVal["OK"]:
            raise WErr.fromSERROR(retVal)
        return {"success": "true", "result": retVal["Value"][0]["Body"]}

    ################################################################################
    def __getLoggingInfo(self, transid):
        tsClient = TransformationClient()
        result = tsClient.getTransformationLogging(transid)
        if result["OK"]:
            result = result["Value"]
            if len(result) > 0:
                callback = []
                resultUser = gConfig.getSections("/Security/Users")
                dndb = {}
                if resultUser["OK"]:
                    users = resultUser["Value"]
                    for j in users:
                        dndb[gConfig.getValue("/Security/Users/%s/DN" % j)] = j
                for i in result:
                    DN = i["AuthorDN"]
                    i["AuthorDN"] = dndb.get(DN, DN)
                    date = Time.toString(i["MessageDate"])
                    callback.append([i["Message"], date, i["AuthorDN"]])
                callback = {"success": "true", "result": callback}
            else:
                callback = {"success": "false", "error": "Nothing to display"}
        else:
            callback = {"success": "false", "error": result["Message"]}
        return callback

    ################################################################################
    def __transformationFileStatus(self, transid):
        tsClient = TransformationClient()
        res = tsClient.getTransformationFilesCount(transid, "Status")
        if not res["OK"]:
            return {"success": "false", "error": res["Message"]}
        resList = []
        total = res["Value"].pop("Total")
        if total == 0:
            return {"success": "false", "error": "No files found"}
        for status in sorted(res["Value"]):
            count = res["Value"][status]
            percent = "%.1f" % ((count * 100.0) / total)
            resList.append((status, str(count), percent))
        resList.append(("Total", total, "-"))
        gLogger.debug("#######", res)
        return {"success": "true", "result": resList}

    ################################################################################
    def __transformationDetail(self, prodid):
        callback = {}

        tsClient = TransformationClient()
        res = tsClient.getTransformationParameters(prodid, ["DetailedInfo"])

        if not res["OK"]:
            callback = {"success": "false", "error": res["Message"]}
        else:
            callback = res["Value"]
            if callback:
                callback = {"success": "true", "result": res["Value"]}
            else:
                callback = {"success": "false", "error": "Production does not have parameter 'DetailedInfo'"}
        gLogger.debug("#######", res)
        return callback

    ################################################################################
    def __extendTransformation(self, transid):
        tasks = int(self.get_argument("tasks"))
        gLogger.info("extend %s" % transid)
        tsClient = TransformationClient()
        gLogger.info("extendTransformation(%s,%s)" % (transid, tasks))
        res = tsClient.extendTransformation(transid, tasks)
        if res["OK"]:
            resString = "%s extended by %s successfully" % (transid, tasks)
        else:
            resString = "%s failed to extend: %s" % (transid, res["Message"])
        callback = {"success": "true", "showResult": [resString], "result": resString}
        gLogger.debug("#######", res)
        return callback

    ################################################################################
    @asyncGen
    def web_showFileStatus(self):
        callback = {}
        start = int(self.get_argument("start"))
        limit = int(self.get_argument("limit"))
        transid = self.get_argument("transformationId")
        status = self.get_argument("status")

        tsClient = TransformationClient()
        result = yield self.threadTask(
            tsClient.getTransformationFilesSummaryWeb,
            {"TransformationID": transid, "Status": status},
            [["FileID", "ASC"]],
            start,
            limit,
        )

        if not result["OK"]:
            callback = {"success": "false", "error": result["Message"]}
        else:
            result = result["Value"]
            if "TotalRecords" in result and result["TotalRecords"] > 0:
                if "ParameterNames" in result and "Records" in result:
                    if len(result["ParameterNames"]) > 0:
                        if len(result["Records"]) > 0:
                            callback = []
                            jobs = result["Records"]
                            head = result["ParameterNames"]
                            headLength = len(head)
                            for i in jobs:
                                tmp = {head[j]: i[j] for j in range(headLength)}
                                callback.append(tmp)
                            total = result["TotalRecords"]
                            timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
                            if "Extras" in result:
                                extra = result["Extras"]
                                callback = {
                                    "success": "true",
                                    "result": callback,
                                    "total": total,
                                    "extra": extra,
                                    "date": timestamp,
                                }
                            else:
                                callback = {"success": "true", "result": callback, "total": total, "date": timestamp}
                        else:
                            callback = {"success": "false", "result": "", "error": "There are no data to display"}
                    else:
                        callback = {"success": "false", "result": "", "error": "ParameterNames field is undefined"}
                else:
                    callback = {"success": "false", "result": "", "error": "Data structure is corrupted"}
            else:
                callback = {"success": "false", "result": "", "error": "There were no data matching your selection"}
        self.finish(callback)

    ################################################################################
    def web_getTier1Sites(self):
        callback = {}
        tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
        if len(tier1) < 1:
            callback = {"success": False, "errors": "No site defined in the CS!"}
        else:
            callback = {"success": True, "data": tier1}
        self.finish(json.dumps(callback))

    ################################################################################
    @asyncGen
    def web_setSite(self):
        callback = {}
        transID = int(self.get_argument("TransformationId"))
        runID = int(self.get_argument("RunNumber"))
        site = self.get_argument("Site")

        gLogger.info("\033[0;31m setTransformationRunsSite(%s, %s, %s) \033[0m" % (transID, runID, site))

        tsClient = TransformationClient()
        result = yield self.threadTask(tsClient.setTransformationRunsSite, transID, runID, site)

        if result["OK"]:
            callback = {"success": "true", "result": "true"}
        else:
            callback = {"success": "false", "error": result["Message"]}
        self.finish(callback)

    ################################################################################
    def _request(self):
        req = {}
        self.numberOfJobs = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))

        prods = list(json.loads(self.get_argument("transformationId", "[]")))
        if prods:
            req["TransformationID"] = prods

        requests = list(json.loads(self.get_argument("requestId", "[]")))
        if requests:
            req["TransformationFamily"] = requests

        if "TransformationFamily" in self.request.arguments:
            req["TransformationFamily"] = self.get_argument("TransformationFamily")

        agentType = list(json.loads(self.get_argument("agentType", "[]")))
        if agentType:
            req["agentType"] = agentType

        status = list(json.loads(self.get_argument("status", "[]")))
        if status:
            req["Status"] = status

        plugin = list(json.loads(self.get_argument("plugin", "[]")))
        if plugin:
            req["Plugin"] = plugin

        transtype = list(json.loads(self.get_argument("type", "[]")))
        if transtype:
            req["Type"] = transtype

        group = list(json.loads(self.get_argument("transformationGroup", "[]")))
        if group:
            req["TransformationGroup"] = group

        if "sort" in self.request.arguments:
            sort = json.loads(self.get_argument("sort"))
            if sort:
                self.globalSort = [["TransformationFamily", "ASC"]]
                for i in sort:
                    self.globalSort += [[i["property"], i["direction"]]]
        else:
            self.globalSort = [["TransformationID", "DESC"]]

        if self.get_argument("startDate", None):
            req["FromDate"] = self.get_argument("startDate")
            if self.get_argument("startTime", None):
                req["FromDate"] += " " + self.get_argument("startTime")

        if self.get_argument("endDate", None):
            req["ToDate"] = self.get_argument("endDate")
            if self.get_argument("endTime", None):
                req["ToDate"] += " " + self.get_argument("endTime")

        if self.get_argument("date", None):
            req["LastUpdate"] = self.get_argument("date")
        gLogger.verbose("REQUEST:", req)
        return req



import os
import json
import datetime
import tempfile

from DIRAC import gConfig, S_OK, S_ERROR
from DIRAC.Core.Utilities import Time, List, DictCache
from DIRAC.Core.Utilities.Plotting.FileCoding import extractRequestFromFileId, codeRequestInFileId
from DIRAC.AccountingSystem.Client.ReportsClient import ReportsClient
from DIRAC.Core.DISET.TransferClient import TransferClient

from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler, FileResponse


class AccountingHandler(WebHandler):
    """This class is the background for the application that shows information about the use of resources at all times."""

    DEFAULT_AUTHORIZATION = "all"

    # Cache of the accounting data
    # Key: (user, group, setup, accounting type)
    __keysCache = DictCache.DictCache()

    # Reports client
    repClient = None
    # Transfer client
    transClient = None

    @classmethod
    def initializeHandler(cls, serviceInfo):
        """This may be overwritten when you write a DIRAC service handler
        And it must be a class method. This method is called only one time,
        at the first request

        :param dict serviceInfo: infos about service
        """
        cls.repClient = ReportsClient()
        cls.transClient = TransferClient("Accounting/ReportGenerator")

    def __getUniqueKeyValues(self, typeName):
        """Get unique key values for accounting type

        :param str typeName: accounting type

        :return: S_OK()/S_ERROR()
        """
        cacheKey = (self.getUserName(), self.getUserGroup(), self.getUserSetup(), typeName)
        data = AccountingHandler.__keysCache.get(cacheKey)
        if not data:
            retVal = self.repClient.listUniqueKeyValues(typeName)
            if "rpcStub" in retVal:
                del retVal["rpcStub"]
            if not retVal["OK"]:
                return retVal

            # Site ordering based on TierLevel / alpha
            if "Site" in retVal["Value"]:
                siteLevel = {}
                for siteName in retVal["Value"]["Site"]:
                    sitePrefix = siteName.split(".")[0].strip()
                    level = gConfig.getValue("/Resources/Sites/%s/%s/MoUTierLevel" % (sitePrefix, siteName), 10)
                    if level not in siteLevel:
                        siteLevel[level] = []
                    siteLevel[level].append(siteName)
                orderedSites = []
                for level in sorted(siteLevel):
                    orderedSites.extend(sorted(siteLevel[level]))
                retVal["Value"]["Site"] = orderedSites
            if "JobGroup" in retVal["Value"]:
                productions = retVal["Value"]["JobGroup"]
                productions.sort(reverse=True)
                retVal["Value"]["JobGroup"] = productions
            data = retVal
            AccountingHandler.__keysCache.add(cacheKey, 300, data)
        return data

    def web_getSelectionData(self, type, **kwargs):
        """Get selection data

        :param type: type of accounting data

        :return: dict
        """
        callback = {}
        typeName = type
        # Get unique key values
        retVal = self.__getUniqueKeyValues(typeName)
        if not retVal["OK"]:
            return {"success": "false", "result": "", "error": retVal["Message"]}

        records = {}
        for record in retVal["Value"]:  # may have more than 1000 of records.
            # do not show all of them in the web portal
            # length = len( retVal['Value'][record] )
            # if  length > 10000:
            #  records[record] = retVal['Value'][record][length - 5000:]
            #  message = "The %s accounting type contains to many rows: %s - > %d. Note: Only 1000 rows are returned!"
            #  message = message % ( typeName, record, length )
            #  gLogger.warn( message )
            # else:
            records[record] = retVal["Value"][record]
        callback["selectionValues"] = records

        # Cache for plotsList?
        data = AccountingHandler.__keysCache.get("reportsList:%s" % typeName)
        if not data:
            retVal = self.repClient.listReports(typeName)
            if not retVal["OK"]:
                return {"success": "false", "result": "", "error": retVal["Message"]}
            data = retVal["Value"]
            AccountingHandler.__keysCache.add("reportsList:%s" % typeName, 300, data)
        callback["plotsList"] = data
        return {"success": "true", "result": callback}

    def __parseFormParams(self, timeSelector, typeName=None, **pD):
        """Prepare parameters

        :param int timeSelector: time selector
        :param str typeName: type of accounting data

        :return: S_OK(tuple)/S_ERROR()
        """
        extraParams = {}

        # Personalized title?
        if "plotTitle" in pD:
            extraParams["plotTitle"] = pD["plotTitle"]
            del pD["plotTitle"]

        # Pin dates?
        pinDates = False
        if "pinDates" in pD:
            pinDates = pD["pinDates"].lower() in ("yes", "y", "true", "1")
            del pD["pinDates"]

        # Find the proper time!
        pD["timeSelector"] = timeSelector
        if pD["timeSelector"] > 0:
            end = Time.dateTime()
            start = end - datetime.timedelta(seconds=pD["timeSelector"])
            if not pinDates:
                extraParams["lastSeconds"] = pD["timeSelector"]
        else:

            end = False
            if "endTime" in pD:
                end = Time.fromString(pD["endTime"])
                del pD["endTime"]
            if "startTime" not in pD:
                return S_ERROR("Missing starTime!")
            start = Time.fromString(pD["startTime"])
            del pD["startTime"]
        del pD["timeSelector"]

        # Find extra parameters
        for k in pD:
            if k.find("ex_") == 0:
                extraParams[k[3:]] = pD[k]

        # Selection data
        data = {}

        # Listify the rest
        for selName in pD:
            if isinstance(pD[selName], str):
                try:
                    pD[selName] = json.loads(pD[selName])
                except ValueError:
                    pD[selName] = List.fromChar(pD[selName], ",")

            # If json parse value as string, listify it
            if isinstance(pD[selName], str):
                pD[selName] = List.fromChar(pD[selName], ",")

            # Convert 'value*' to list of values that starts with 'value'
            fullList = []
            for value in pD[selName]:
                if value.endswith("*"):
                    if not data:
                        retVal = self.__getUniqueKeyValues(typeName)
                        if not retVal["OK"]:
                            return retVal
                        data = retVal["Value"]
                    for v in data[selName]:
                        if v.startswith(value[:-1]):
                            fullList.append(v)
                else:
                    fullList.append(value)

            pD[selName] = fullList

        return S_OK((start, end, pD, extraParams))

    def web_generatePlot(self, typeName, plotName, timeSelector: int, grouping, **kwargs):
        """Generate plot

        :param str typeName: accounting type name
        :param str plotName: plot name
        :param int timeSelector: time selector value
        :param str grouping: grouping field name

        :return: dict
        """
        retVal = self.__parseFormParams(timeSelector, grouping=[grouping], typeName=typeName, **kwargs)
        if retVal["OK"]:
            start, end, pD, kwargs = retVal["Value"]
            retVal = self.repClient.generateDelayedPlot(typeName, plotName, start, end, pD, grouping, kwargs)
        if retVal["OK"]:
            return {"success": True, "data": retVal["Value"]["plot"]}
        return {"success": False, "errors": retVal["Message"]}

    def web_getPlotImg(self, file=None, **kwargs):
        """Get plot image

        :param str file: file path

        :return: dict
        """
        plotImageFile = file
        if not plotImageFile:
            return {"success": "false", "error": "Maybe you forgot the file?"}
        # Prevent directory traversal
        plotImageFile = os.path.normpath("/" + plotImageFile).lstrip("/")

        tempFile = tempfile.TemporaryFile()
        retVal = self.transClient.receiveFile(tempFile, plotImageFile)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Message"]}

        tempFile.seek(0)
        data = tempFile.read()
        return FileResponse(data, plotImageFile, "png")

    def web_getPlotImgFromCache(self, file=None, **kwargs):
        """Get plot image from cache.

        :param str file: file path

        :return: dict
        """
        plotImageFile = file
        if not plotImageFile:
            return {"success": "false", "error": "Maybe you forgot the file?"}

        retVal = extractRequestFromFileId(plotImageFile)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Value"]}
        fields = retVal["Value"]
        if "extraArgs" in fields:  # in order to get the plot from the cache we have to clean the extraArgs...
            plotTitle = fields["extraArgs"].get("plotTitle", "")
            fields["extraArgs"] = {}
            if plotTitle:
                fields["extraArgs"]["plotTitle"] = plotTitle

        retVal = codeRequestInFileId(fields)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Value"]}

        plotImageFile = retVal["Value"]["plot"]
        tempFile = tempfile.TemporaryFile()
        retVal = self.transClient.receiveFile(tempFile, plotImageFile)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Message"]}

        tempFile.seek(0)
        data = tempFile.read()
        return FileResponse(data, plotImageFile, "png", cache=False)

    def web_getCsvPlotData(self, typeName, plotName, timeSelector: int, grouping, **kwargs):
        """Generate CVS plot

        :param str typeName: accounting type name
        :param str plotName: plot name
        :param int timeSelector: time selector value
        :param str grouping: grouping field name

        :return: dict
        """
        retVal = self.__parseFormParams(timeSelector, grouping=[grouping], typeName=typeName, **kwargs)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Message"]}
        start, end, pD, kwargs = retVal["Value"]
        params = (typeName, plotName, start, end, pD, grouping, kwargs)
        retVal = self.repClient.getReport(*params)
        if not retVal["OK"]:
            return {"success": "false", "error": retVal["Message"]}

        rawData = retVal["Value"]
        groupKeys = sorted(rawData["data"])
        if "granularity" in rawData:
            granularity = rawData["granularity"]
            data = rawData["data"]
            tS = int(Time.toEpoch(start))
            timeStart = tS - tS % granularity
            strData = "epoch,%s\n" % ",".join(groupKeys)
            for timeSlot in range(timeStart, int(Time.toEpoch(end)), granularity):
                lineData = [str(timeSlot)]
                for key in groupKeys:
                    lineData.append(str(data[key][timeSlot]) if timeSlot in data[key] else "")
                strData += "%s\n" % ",".join(lineData)
        else:
            strData = "%s\n" % ",".join(groupKeys)
            strData += ",".join([str(rawData["data"][k]) for k in groupKeys])

        return FileResponse(strData, str(params), "csv", cache=False)

    def web_getPlotData(self, typeName, plotName, timeSelector: int, grouping, **kwargs):
        """Generate plot

        :param str typeName: accounting type name
        :param str plotName: plot name
        :param int timeSelector: time selector value
        :param str grouping: grouping field name

        :return: dict
        """
        retVal = self.__parseFormParams(timeSelector, grouping=[grouping], typeName=typeName, **kwargs)
        if retVal["OK"]:
            start, end, pD, kwargs = retVal["Value"]
            retVal = self.repClient.getReport(typeName, plotName, start, end, pD, grouping, kwargs)

        return retVal["Value"] if retVal["OK"] else {"success": "false", "error": retVal["Message"]}



#!/usr/bin/env python

import os
import sys

from DIRAC.Core.Utilities.DIRACScript import DIRACScript


@DIRACScript()
def main():
    # Must be define BEFORE any dirac import.
    os.environ["DIRAC_USE_TORNADO_IOLOOP"] = "True"

    from DIRAC.ConfigurationSystem.Client.LocalConfiguration import LocalConfiguration
    from DIRAC.Core.Tornado.Server.TornadoServer import TornadoServer
    from DIRAC.Core.Utilities.DErrno import includeExtensionErrors
    from DIRAC.FrameworkSystem.Client.Logger import gLogger

    localCfg = LocalConfiguration()
    localCfg.addMandatoryEntry("/DIRAC/Setup")
    localCfg.addDefaultEntry("/DIRAC/Security/UseServerCertificate", "yes")
    localCfg.addDefaultEntry("LogLevel", "INFO")
    localCfg.addDefaultEntry("LogColor", True)
    resultDict = localCfg.loadUserData()
    if not resultDict["OK"]:
        gLogger.initialize("Tornado", "/")
        gLogger.error("There were errors when loading configuration", resultDict["Message"])
        sys.exit(1)

    includeExtensionErrors()

    gLogger.initialize("Tornado", "/")

    serverToLaunch = TornadoServer(False)

    try:
        from WebAppDIRAC.Core.App import App
    except ImportError as e:
        gLogger.fatal("Web portal is not installed. %s" % repr(e))
        sys.exit(1)

    # Get routes and settings for a portal
    result = App().getAppToDict(8010)
    if not result["OK"]:
        gLogger.fatal(result["Message"])
        sys.exit(1)
    app = result["Value"]

    serverToLaunch.addHandlers(app["routes"], app["settings"], app["port"])

    serverToLaunch.startTornado()


if __name__ == "__main__":
    main()



import pprint
import tempfile

from DIRAC import gConfig, gLogger
from DIRAC.Resources.Catalog.FileCatalog import FileCatalog
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getVOForGroup
from DIRAC.ConfigurationSystem.Client.Helpers.Operations import Operations

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class JobLaunchpadHandler(WebHandler):

    AUTH_PROPS = "authenticated"
    defaultParams = {
        "JobName": [1, "DIRAC"],
        "Executable": [1, "/bin/ls"],
        "Arguments": [1, "-ltrA"],
        "OutputSandbox": [1, "std.out, std.err"],
        "JobGroup": [0, "Unknown"],
        "InputData": [0, ""],
        "OutputData": [0, ""],
        "OutputSE": [0, "DIRAC-USER"],
        "OutputPath": [0, ""],
        "CPUTime": [0, "86400"],
        "Site": [0, ""],
        "BannedSite": [0, ""],
        "Platform": [0, "Linux_x86_64_glibc-2.12"],
        "Priority": [0, "5"],
        "StdError": [0, "std.err"],
        "StdOutput": [0, "std.out"],
        "Parameters": [0, "0"],
        "ParameterStart": [0, "0"],
        "ParameterStep": [0, "1"],
        "ParameterFactor": [0, "0"],
    }

    def initializeRequest(self):
        for opt, value in (self.getAppSettings().get("Options") or {}).items():
            self.defaultParams[opt] = value.replace(", ", ",").split(",")
        self.user = self.getUserName()
        self.group = self.getUserGroup()
        self.vo = getVOForGroup(self.group)

    def web_getProxyStatus(self):
        self.finish(self.__getProxyStatus())

    def __getProxyStatus(self, secondsOverride=None):
        from DIRAC.FrameworkSystem.Client.ProxyManagerClient import ProxyManagerClient

        proxyManager = ProxyManagerClient()

        group = self.getUserGroup()

        if group == "visitor":
            return {"success": "false", "error": "User is anonymous or is not registered in the system"}

        userDN = self.getUserDN()

        defaultSeconds = 24 * 3600 + 60  # 24H + 1min
        validSeconds = gConfig.getValue("/Registry/DefaultProxyLifeTime", defaultSeconds)

        gLogger.info("\033[0;31m userHasProxy(%s, %s, %s) \033[0m" % (userDN, group, validSeconds))

        result = proxyManager.userHasProxy(userDN, group, validSeconds)

        if result["OK"]:
            if result["Value"]:
                return {"success": "true", "result": "true"}
            else:
                return {"success": "true", "result": "false"}
        else:
            return {"success": "false", "error": "false"}

        gLogger.info("\033[0;31m PROXY: \033[0m", result)

    def __getPlatform(self):
        gLogger.info("start __getPlatform")

        path = "/Resources/Computing/OSCompatibility"
        result = gConfig.getOptionsDict(path)

        gLogger.debug(result)

        if not result["OK"]:
            return False

        platformDict = result["Value"]
        platform = list(platformDict)

        gLogger.debug("platform: %s" % platform)
        gLogger.info("end __getPlatform")
        return platform

    @asyncGen
    def web_getLaunchpadSetupWithLFNs(self):
        """Method obtain launchpad setup with pre-selected LFNs as input data parameter,
        the caller js client will use setup to open an new Launchpad
        """
        # On the fly file catalog for advanced launchpad
        if not hasattr(self, "fc"):
            group = self.getUserGroup()
            vo = getVOForGroup(group)
            self.fc = FileCatalog(vo=vo)

        self.set_header("Content-type", "text/plain")
        gLogger.always("submit: incoming arguments to getLaunchpadSetupWithLFNs", repr(self.request.arguments))
        lfnList = self.get_argument("path").split(",")

        ptlfn = ""
        for lfn in lfnList:
            ptlfn += (", " + lfn) if ptlfn else lfn

        params = self.defaultParams.copy()
        params["InputData"] = [1, ptlfn]

        obj = Operations(vo=vo)
        predefinedSets = {}
        launchpadSections = obj.getSections("Launchpad")
        if launchpadSections["OK"]:
            for section in launchpadSections["Value"]:
                predefinedSets[section] = {}
                sectionOptions = obj.getOptionsDict("Launchpad/" + section)
                pprint.pprint(sectionOptions)
                if sectionOptions["OK"]:
                    predefinedSets[section] = sectionOptions["Value"]

        self.finish({"success": "true", "result": params, "predefinedSets": predefinedSets})

    def web_getLaunchpadOpts(self):
        """Reading of the predefined sets of launchpad parameters values"""
        obj = Operations(vo=self.vo)
        predefinedSets = {}
        launchpadSections = obj.getSections("Launchpad")
        if launchpadSections["OK"]:
            for section in obj.getValue("Launchpad/ApplicationList", launchpadSections["Value"]):
                predefinedSets[section] = {}
                sectionOptions = obj.getOptionsDict("Launchpad/" + section)
                pprint.pprint(sectionOptions)
                if sectionOptions["OK"]:
                    predefinedSets[section] = sectionOptions["Value"]
        self.finish({"success": "true", "result": self.defaultParams, "predefinedSets": predefinedSets})

    @asyncGen
    def web_jobSubmit(self):
        # self.set_header('Content-type', "text/html")  # Otherwise the browser would offer you to download a JobSubmit file
        if "NormalUser" not in self.getProperties():
            self.finish({"success": "false", "error": "You are not allowed to run the jobs"})
            return
        proxy = yield self.threadTask(self.__getProxyStatus, 86460)
        if proxy["success"] == "false" or proxy["result"] == "false":
            self.finish({"success": "false", "error": "You can not run a job: your proxy is valid less then 24 hours"})
            return

        jdl = ""
        lfns = []
        params = {}
        for tmp in self.request.arguments:
            value = self.get_argument(tmp)
            if value:
                if tmp.startswith("lfnField"):
                    lfns.append("LFN:" + value)
                else:
                    params[tmp] = value

        for item in params:
            if item == "OutputSandbox":
                jdl += str(item) + " = {" + str(params[item]) + "};"
            if item == "Parameters":
                try:
                    parameters = int(params[item])
                    jdl += str(item) + ' = "' + str(parameters) + '";'
                except Exception:
                    parameters = str(params[item])
                    if parameters.find("{") >= 0 and parameters.find("}") >= 0:
                        parameters = parameters.rstrip("}")
                        parameters = parameters.lstrip("{")
                        if len(parameters) > 0:
                            jdl += str(item) + " = {" + parameters + "};"
                        else:
                            self.finish({"success": "false", "error": "Parameters vector has zero length"})
                            return
                    else:
                        self.finish(
                            {
                                "success": "false",
                                "error": "Parameters must be an integer or a vector. Example: 4 or {1,2,3,4}",
                            }
                        )
                        return
            else:
                jdl += str(item) + ' = "' + str(params[item]) + '";'

        store = []
        for key in self.request.files:
            try:
                if self.request.files[key][0].filename:
                    gLogger.info("\033[0;31m file - %s \033[0m " % self.request.files[key][0].filename)
                    store.append(self.request.files[key][0])
            except Exception:
                pass

        gLogger.info("\033[0;31m *** %s \033[0m " % params)

        clearFS = False  # Clear directory flag
        fileNameList = []
        exception_counter = 0
        callback = {}

        if len(store) > 0:  # If there is a file(s) in sandbox
            clearFS = True
            import shutil
            import os

            storePath = tempfile.mkdtemp(prefix="DIRAC_")
            try:
                for fileObj in store:
                    name = os.path.join(storePath, fileObj.filename.lstrip(os.sep))

                    tFile = open(name, "w")
                    tFile.write(fileObj.body)
                    tFile.close()

                    fileNameList.append(name)
            except Exception as x:
                exception_counter = 1
                callback = {
                    "success": "false",
                    "error": "An EXCEPTION happens during saving your sandbox file(s): %s" % str(x),
                }

        sndBox = ""
        if ((len(fileNameList) > 0) or (len(lfns) > 0)) and exception_counter == 0:
            sndBox = 'InputSandbox = {"' + '","'.join(fileNameList + lfns) + '"};'

        if exception_counter == 0:
            jdl += sndBox
            from DIRAC.WorkloadManagementSystem.Client.WMSClient import WMSClient

            jobManager = WMSClient(useCertificates=True, timeout=1800)
            jdl = str(jdl)
            gLogger.info("J D L : ", jdl)
            try:
                result = yield self.threadTask(jobManager.submitJob, jdl)
                if result["OK"]:
                    callback = {"success": "true", "result": result["Value"]}
                else:
                    callback = {"success": "false", "error": result["Message"]}
            except Exception as x:
                callback = {"success": "false", "error": "An EXCEPTION happens during job submittion: %s" % str(x)}
        if clearFS:
            shutil.rmtree(storePath)
        self.finish(callback)



import os
import json
import tempfile
import datetime

from hashlib import md5

from DIRAC import gConfig, S_OK, S_ERROR, gLogger
from DIRAC.Core.DISET.TransferClient import TransferClient
from DIRAC.Core.Utilities import Time, List, DictCache
from DIRAC.Core.Utilities.Plotting.FileCoding import extractRequestFromFileId, codeRequestInFileId
from DIRAC.MonitoringSystem.Client.MonitoringClient import MonitoringClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class MonitoringHandler(WebHandler):

    AUTH_PROPS = "authenticated"
    __keysCache = DictCache.DictCache()

    def __getUniqueKeyValues(self, typeName):
        cacheKey = (self.getUserName(), self.getUserGroup(), self.getUserSetup(), typeName)
        data = MonitoringHandler.__keysCache.get(cacheKey)
        if not data:
            client = MonitoringClient()
            retVal = client.listUniqueKeyValues(typeName)
            if "rpcStub" in retVal:
                del retVal["rpcStub"]
            if not retVal["OK"]:
                return retVal

            # Site ordering based on TierLevel / alpha
            if "Site" in retVal["Value"]:
                siteLevel = {}
                for siteName in retVal["Value"]["Site"]:
                    sitePrefix = siteName.split(".")[0].strip()
                    level = gConfig.getValue("/Resources/Sites/%s/%s/MoUTierLevel" % (sitePrefix, siteName), 10)
                    if level not in siteLevel:
                        siteLevel[level] = []
                    siteLevel[level].append(siteName)
                orderedSites = []
                for level in sorted(siteLevel):
                    orderedSites.extend(sorted(siteLevel[level]))
                retVal["Value"]["Site"] = orderedSites
            data = retVal
            MonitoringHandler.__keysCache.add(cacheKey, 300, data)
        return data

    @asyncGen
    def web_getSelectionData(self):
        callback = {}
        typeName = self.get_argument("type")
        # Get unique key values
        retVal = yield self.threadTask(self.__getUniqueKeyValues, typeName)
        if not retVal["OK"]:
            self.finish({"success": "false", "result": "", "error": retVal["Message"]})
            return

        records = {}
        for record in retVal["Value"]:  # may have more than 1000 of records.
            # do not show all of them in the web portal
            length = len(retVal["Value"][record])
            if length > 10000:
                records[record] = retVal["Value"][record][length - 5000 :]
                message = (
                    "The %s accounting type contains to many rows: %s - > %d. Note: Only 1000 rows are returned!"
                    % (typeName, record, length)
                )
                gLogger.warn(message)
            else:
                records[record] = retVal["Value"][record]
        callback["selectionValues"] = records

        # Cache for plotsList?
        data = MonitoringHandler.__keysCache.get("reportsList:%s" % typeName)
        if not data:
            repClient = MonitoringClient()
            retVal = yield self.threadTask(repClient.listReports, typeName)
            if not retVal["OK"]:
                self.finish({"success": "false", "result": "", "error": retVal["Message"]})
                return
            data = retVal["Value"]
            MonitoringHandler.__keysCache.add("reportsList:%s" % typeName, 300, data)
        callback["plotsList"] = data
        self.finish({"success": "true", "result": callback})

    def __parseFormParams(self):
        pD = {}
        extraParams = {}
        pinDates = False

        for name in self.request.arguments:
            pD[name] = self.get_argument(name)

        # Personalized title?
        if "plotTitle" in pD:
            extraParams["plotTitle"] = pD["plotTitle"]
            del pD["plotTitle"]
        # Pin dates?
        if "pinDates" in pD:
            pinDates = pD["pinDates"]
            del pD["pinDates"]
            pinDates = pinDates.lower() in ("yes", "y", "true", "1")
        # Get plotname
        if "grouping" not in pD:
            return S_ERROR("Missing grouping!")
        grouping = pD["grouping"]
        # Get plotname
        if "typeName" not in pD:
            return S_ERROR("Missing type name!")
        typeName = pD["typeName"]
        del pD["typeName"]
        # Get plotname
        if "plotName" not in pD:
            return S_ERROR("Missing plot name!")
        reportName = pD["plotName"]
        del pD["plotName"]
        # Get times
        if "timeSelector" not in pD:
            return S_ERROR("Missing time span!")
        # Find the proper time!
        pD["timeSelector"] = int(pD["timeSelector"])
        if pD["timeSelector"] > 0:
            end = Time.dateTime()
            start = end - datetime.timedelta(seconds=pD["timeSelector"])
            if not pinDates:
                extraParams["lastSeconds"] = pD["timeSelector"]
        else:
            if "endTime" not in pD:
                end = False
            else:
                end = Time.fromString(pD["endTime"])
                del pD["endTime"]
            if "startTime" not in pD:
                return S_ERROR("Missing starTime!")
            else:
                start = Time.fromString(pD["startTime"])
                del pD["startTime"]
        del pD["timeSelector"]

        for k in pD:
            if k.find("ex_") == 0:
                extraParams[k[3:]] = pD[k]

        # Selection data
        data = {}

        # Listify the rest
        for selName in pD:
            if selName == "grouping":
                pD[selName] = [pD[selName]]
            else:
                try:
                    pD[selName] = json.loads(pD[selName])
                except ValueError:
                    pD[selName] = List.fromChar(pD[selName], ",")

            # If json parse value as string, listify it
            if isinstance(pD[selName], str):
                pD[selName] = List.fromChar(pD[selName], ",")

            # Convert 'value*' to list of values that starts with 'value'
            fullList = []
            for value in pD[selName]:
                if value.endswith("*"):
                    if not data:
                        retVal = self.__getUniqueKeyValues(typeName)
                        if not retVal["OK"]:
                            return retVal
                        data = retVal["Value"]
                    for v in data[selName]:
                        if v.startswith(value[:-1]):
                            fullList.append(v)
                else:
                    fullList.append(value)

            pD[selName] = fullList

        return S_OK((typeName, reportName, start, end, pD, grouping, extraParams))

    @asyncGen
    def web_generatePlot(self):
        callback = {}
        retVal = yield self.threadTask(self.__queryForPlot)
        if retVal["OK"]:
            callback = {"success": True, "data": retVal["Value"]["plot"]}
        else:
            callback = {"success": False, "errors": retVal["Message"]}
        self.finish(callback)

    def __queryForPlot(self):
        res = self.__parseFormParams()
        return MonitoringClient().generateDelayedPlot(*res["Value"]) if res["OK"] else res

    @asyncGen
    def web_getPlotImg(self):
        """
        Get plot image
        """
        callback = {}
        if "file" not in self.request.arguments:
            callback = {"success": "false", "error": "Maybe you forgot the file?"}
            self.finish(callback)
            return
        plotImageFile = self.get_argument("file")
        # Prevent directory traversal
        plotImageFile = os.path.normpath("/" + plotImageFile).lstrip("/")

        transferClient = TransferClient("Monitoring/Monitoring")
        tempFile = tempfile.TemporaryFile()
        retVal = yield self.threadTask(transferClient.receiveFile, tempFile, plotImageFile)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
            return
        tempFile.seek(0)
        data = tempFile.read()
        self.finishWithImage(data, plotImageFile)

    @asyncGen
    def web_getPlotImgFromCache(self):
        """
        Get plot image from cache.
        """
        callback = {}
        if "file" not in self.request.arguments:
            callback = {"success": "false", "error": "Maybe you forgot the file?"}
            self.finish(callback)
            return
        plotImageFile = self.get_argument("file")

        retVal = extractRequestFromFileId(plotImageFile)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Value"]}
            self.finish(callback)
            return
        fields = retVal["Value"]
        if "extraArgs" in fields:  # in order to get the plot from the cache we have to clean the extraArgs...
            plotTitle = ""
            if "plotTitle" in fields["extraArgs"]:
                plotTitle = fields["extraArgs"]["plotTitle"]
                fields["extraArgs"] = {}
                fields["extraArgs"]["plotTitle"] = plotTitle
            else:
                fields["extraArgs"] = {}

        retVal = codeRequestInFileId(fields)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Value"]}
            self.finish(callback)
            return
        plotImageFile = retVal["Value"]["plot"]

        transferClient = TransferClient("Monitoring/Monitoring")
        tempFile = tempfile.TemporaryFile()
        retVal = yield self.threadTask(transferClient.receiveFile, tempFile, plotImageFile)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
            return
        tempFile.seek(0)
        data = tempFile.read()
        self.finishWithImage(data, plotImageFile, disableCaching=True)

    @asyncGen
    def web_getCsvPlotData(self):
        callback = {}
        retVal = self.__parseFormParams()
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
        params = retVal["Value"]
        repClient = MonitoringClient()
        retVal = yield self.threadTask(repClient.getReport, *params)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
        rawData = retVal["Value"]
        groupKeys = sorted(rawData["data"])
        if "granularity" in rawData:
            granularity = rawData["granularity"]
            data = rawData["data"]
            tS = int(Time.toEpoch(params[2]))
            timeStart = tS - tS % granularity
            strData = "epoch,%s\n" % ",".join(groupKeys)
            for timeSlot in range(timeStart, int(Time.toEpoch(params[3])), granularity):
                lineData = [str(timeSlot)]
                for key in groupKeys:
                    if timeSlot in data[key]:
                        lineData.append(str(data[key][timeSlot]))
                    else:
                        lineData.append("")
                strData += "%s\n" % ",".join(lineData)
        else:
            strData = "%s\n" % ",".join(groupKeys)
            strData += ",".join([str(rawData["data"][k]) for k in groupKeys])
        self.set_header("Content-type", "text/csv")
        self.set_header("Content-Disposition", 'attachment; filename="%s.csv"' % md5(str(params).encode()).hexdigest())
        self.set_header("Content-Length", len(strData))
        self.finish(strData)

    @asyncGen
    def web_getPlotData(self):
        callback = {}
        retVal = self.__parseFormParams()
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
        params = retVal["Value"]
        repClient = MonitoringClient()
        retVal = yield self.threadTask(repClient.getReport, *params)
        if not retVal["OK"]:
            callback = {"success": "false", "error": retVal["Message"]}
            self.finish(callback)
        rawData = retVal["Value"]
        self.finish(rawData["data"])



from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen
from DIRAC.FrameworkSystem.Client import ProxyUpload
from DIRAC.Core.Security.X509Chain import X509Chain  # pylint: disable=import-error
from DIRAC import gLogger
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getGroupsForDN


class ProxyUploadHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_proxyUpload(self):
        """
        Get p12 file and passwords as input. Split p12 to user key and certificate
        and creating proxy for groups user belongs to. Upload proxy to proxy store
        """
        # Otherwise the browser would offer to download a file
        # response.headers['Content-type'] = "text/html"
        userData = self.getSessionData()
        username = userData["user"]["username"]
        gLogger.info("Start upload proxy out of p12 for user: %s" % (username))
        disclaimer = "\nNo proxy was created\nYour private info was safely deleted"
        disclaimer = disclaimer + " from DIRAC service"

        if username == "anonymous":
            error = "Please, send a registration request first"
            gLogger.error("Anonymous is not allowed")
            gLogger.debug("Service response: %s" % error)
            self.finish({"success": "false", "error": error})
            return

        groupList = userData["validGroups"]
        gLogger.info("Available groups for the user %s: %s" % (username, ", ".join(groupList)))

        if not len(groupList) > 0:
            gLogger.error("User is not registered in any group")
            error = "Seems that user %s is not register in any group" % username
            error = error + disclaimer
            gLogger.debug("Service response: %s" % error)
            self.finish({"success": "false", "error": error})
            return

        fileObject = None
        gLogger.debug("Request's body:")
        key = list(self.request.files)[0]
        try:
            if self.request.files[key][0].filename:
                name = self.request.files[key][0].filename
                name = name.strip()
                if name[-4:] == ".p12":
                    gLogger.info(".p12 in filename detected")
                    if self.get_argument("pass_p12", None):
                        fileObject = self.request.files[key][0]
                        fileObject.p12 = self.get_argument("pass_p12")
                        gLogger.info(".p12 password detected")
                        # store.append(fileObject)
                        gLogger.info("Certificate object is loaded")
        except Exception as x:
            gLogger.debug("Non fatal for logic, exception happens: %s" % str(x))
            pass

        if fileObject is None:  # If there is a file(s) to store
            gLogger.error("No file with *.p12 found")
            error = "Failed to find any suitable *.p12 filename in your request"
            error = error + disclaimer
            gLogger.debug("Service response: %s" % error)
            self.finish({"success": "false", "error": error})
            return

        import tempfile
        import shutil
        import os
        import random
        import string

        storePath = tempfile.mkdtemp(prefix="DIRAC_")
        gLogger.info("Saving file from request to a tmp directory")

        try:
            descriptionDict = dict()
            for i in "name", "p12", "pem":
                tmp = "".join(random.choice(string.ascii_letters) for x in range(10))
                descriptionDict[i] = os.path.join(storePath, tmp)

            with open(descriptionDict["name"], "wb") as tmpFile:
                tmpFile.write(fileObject.body)

            with open(descriptionDict["p12"], "w") as tmpFile:
                tmpFile.write(fileObject.p12)

            pemPassword = "".join(random.choice(string.ascii_letters) for x in range(10))

            with open(descriptionDict["pem"], "w") as tmpFile:
                tmpFile.write(pemPassword)
        except Exception as x:
            shutil.rmtree(storePath)
            gLogger.exception(x)
            error = "An exception has happen '%s'" % str(x)
            error = error + disclaimer
            gLogger.debug("Service response: %s" % error)
            self.finish({"success": "false", "error": error})
            return

        gLogger.info("Split certificate(s) to public and private keys")

        from DIRAC.Core.Utilities import Subprocess

        keyDict = dict()

        name = descriptionDict["name"]
        p12 = descriptionDict["p12"]
        keyDict["pem"] = descriptionDict["pem"]

        for j in "pub", "private":
            tmp = "".join(random.choice(string.ascii_letters) for x in range(10))
            keyDict[j] = os.path.join(storePath, tmp)

        cmdCert = "openssl pkcs12 -clcerts -nokeys -in %s -out %s -password file:%s" % (name, keyDict["pub"], p12)
        cmdKey = "openssl pkcs12 -nocerts -in %s -out %s -passout file:%s -password file:%s" % (
            name,
            keyDict["private"],
            keyDict["pem"],
            p12,
        )

        for cmd in cmdCert, cmdKey:
            result = yield self.threadTask(Subprocess.shellCall, 900, cmd)
            gLogger.debug("Command is: %s" % cmd)
            gLogger.debug("Result is: %s" % result)
            if not result["OK"]:
                shutil.rmtree(storePath)
                gLogger.error(result["Message"])
                error = "Error while executing SSL command: %s" % result["Message"]
                error = error + disclaimer
                gLogger.debug("Service response: %s" % error)
                self.finish({"success": "false", "error": error})
                return

        proxyChain = X509Chain()

        result = proxyChain.loadChainFromFile(keyDict["pub"])
        if not result["OK"]:
            self.finish({"error": "Could not load the proxy: %s" % result["Message"], "success": "false"})
            return

        result = proxyChain.getIssuerCert()
        if not result["OK"]:
            self.finish({"error": "Could not load the proxy: %s" % result["Message"], "success": "false"})
            return
        issuerCert = result["Value"]

        upParams = ProxyUpload.CLIParams()
        upParams.onTheFly = True
        upParams.proxyLifeTime = issuerCert.getRemainingSecs()["Value"] - 300
        upParams.certLoc = keyDict["pub"]
        upParams.keyLoc = keyDict["private"]
        upParams.userPasswd = pemPassword
        result = ProxyUpload.uploadProxy(upParams)

        if not result["OK"]:
            self.finish({"error": result["Message"], "success": "false"})
            return

        shutil.rmtree(storePath)

        result = issuerCert.getSubjectDN()
        if result["OK"]:
            result = getGroupsForDN(result["Value"])
        if not result["OK"]:
            self.finish({"error": result["Message"], "success": "false"})
            return
        groups = ", ".join(result["Value"])
        result = "Operation finished successfully\n"
        result += "Proxy uploaded for user: %s \n" % username
        if len(groupList) > 0:
            result += " in groups: %s \n" % groups
        else:
            result += " in group: %s \n" % groups

        result += "\nYour private info was safely deleted from DIRAC server."
        gLogger.info(result)
        self.finish({"success": "true", "result": result})



import json

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.ConfigurationSystem.Client.Helpers.Registry import getAllUsers
from DIRAC.FrameworkSystem.Client.TokenManagerClient import TokenManagerClient

from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler


class TokenManagerHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "authenticated"

    @classmethod
    def initializeHandler(cls, serviceInfo):
        """Init"""
        cls.tm = TokenManagerClient()

    def web_getSelectionData(self, **kwargs):
        user = self.getUserName()
        if user.lower() == "anonymous":
            return {"success": "false", "error": "You are not authorize to access these data"}

        users = getAllUsers()
        users.sort()
        return {"username": [[x] for x in users]}

    def web_getTokenManagerData(self, username="[]", **kwargs):
        """Get tokens information

        :param str username: user name

        :return: dict
        """
        user = self.getUserName()
        if user.lower() == "anonymous":
            return {"success": "false", "error": "You are not authorize to access these data"}

        result = self.tm.getUsersTokensInfo(list(json.loads(username)))
        gLogger.info("*!*!*!  RESULT: \n%s" % result)
        if not result["OK"]:
            return {"success": "false", "error": result["Message"]}

        tokens = []
        for record in result["Value"]:
            tokens.append(
                {
                    "tokenid": record["user_id"],
                    "UserName": record["username"],
                    "UserID": record["user_id"],
                    "Provider": record["provider"],
                    "ExpirationTime": str(record["rt_expires_at"]),
                }
            )
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        return {"success": "true", "result": tokens, "total": len(tokens), "date": timestamp}

    def web_deleteTokens(self, idList):
        """Delete token

        :param str idList: IDs

        :return: dict
        """
        err = []

        try:
            webIds = list(json.loads(idList))
        except Exception:
            return {"success": "false", "error": "No valid id's specified"}

        tokens = []
        for uid in webIds:
            retVal = self.tm.deleteToken(uid)
            if retVal["OK"]:
                tokens.append(uid)
            else:
                err.append(retVal["Message"])
        return {"success": "true", "result": tokens} if tokens else {"success": "false", "error": "; ".join(err)}



""" Handler for SystemAdministration WebApp
"""

import json
import datetime

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities.List import uniqueElements
from DIRAC.FrameworkSystem.Client.MonitoringClient import gMonitor
from DIRAC.FrameworkSystem.Client.NotificationClient import NotificationClient
from DIRAC.FrameworkSystem.Client.SystemAdministratorClient import SystemAdministratorClient
from DIRAC.FrameworkSystem.Client.ComponentMonitoringClient import ComponentMonitoringClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class SystemAdministrationHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getSysInfo(self):
        """Provide information about hosts state from database"""
        DN = self.getUserDN()
        group = self.getUserGroup()
        client = ComponentMonitoringClient(delegatedDN=DN, delegatedGroup=group)
        result = yield self.threadTask(client.getLogs)
        if not result["OK"] or not len(result["Value"]) > 0:
            self.finish({"success": "false", "error": result.get("Message", "No system information found")})
            return

        callback = result["Value"]

        # Add the information about the extensions' versions, if available, to display along with the DIRAC version
        for i in range(len(callback)):
            callback[i]["Host"] = callback[i]["HostName"]
            callback[i]["Timestamp"] = str(callback[i].get("Timestamp", "unknown"))
            # We have to keep the backward compatibility (this can heppen when we do not update one host to v6r15 ...
            callback[i]["DIRAC"] = "%s,%s" % (
                callback[i].get("DIRACVersion", callback[i].get("DIRAC", "")),
                callback[i].get("Extension", callback[i].get("Extensions", "")),
            )

        self.finish({"success": "true", "result": sorted(callback, key=lambda i: i["Host"]), "total": len(callback)})

    @asyncGen
    def web_getHostData(self):
        """
        Returns flatten list of components (services, agents) installed on hosts
        returned by getHosts function
        """
        DN = self.getUserDN()
        group = self.getUserGroup()

        callback = list()

        if "hostname" not in self.request.arguments or not self.get_argument("hostname"):
            self.finish({"success": "true", "result": callback})
            return

        host = self.get_argument("hostname")
        client = SystemAdministratorClient(host, None, delegatedDN=DN, delegatedGroup=group)
        result = yield self.threadTask(client.getOverallStatus)
        gLogger.debug("Result of getOverallStatus(): %s" % result)

        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return

        overall = result["Value"]

        for record in self.flatten(overall):
            record["Host"] = host
            callback.append(record)

        self.finish({"success": "true", "result": callback})

    def flatten(self, dataDict):
        """
        Flatten dict of dicts structure returned by getOverallStatus() method of
        SystemAdministrator client
        """
        for kind, a in dataDict.items():
            for system, b in a.items():
                for name, c in b.items():
                    if "Installed" in c and c["Installed"]:
                        c["Type"] = kind
                        c["System"] = system
                        c["Name"] = name
                        yield c

    @asyncGen
    def web_getHostErrors(self):
        DN = self.getUserDN()
        group = self.getUserGroup()

        if "host" not in self.request.arguments:
            self.finish({"success": "false", "error": "Name of the host is missing or not defined"})
            return

        host = self.get_argument("host")

        client = SystemAdministratorClient(host, None, delegatedDN=DN, delegatedGroup=group)

        result = yield self.threadTask(client.checkComponentLog, "*")

        gLogger.debug(result)
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        result = result["Value"]

        callback = list()
        for key, value in result.items():
            system, component = key.split("/")
            value["System"] = system
            value["Name"] = component
            value["Host"] = host
            callback.append(value)
        total = len(callback)

        self.finish({"success": "true", "result": callback, "total": total})

    @asyncGen
    def web_getHostLog(self):
        DN = self.getUserDN()
        group = self.getUserGroup()

        if "host" not in self.request.arguments:
            self.finish({"success": "false", "error": "Name of the host is missing or not defined"})
            return
        host = self.get_argument("host")

        if "system" not in self.request.arguments:
            self.finish({"success": "false", "error": "Name of the system is missing or not defined"})
            return
        system = self.get_argument("system")

        if "component" not in self.request.arguments:
            self.finish({"success": "false", "error": "Name of component is missing or not defined"})
            return

        name = self.get_argument("component")

        client = SystemAdministratorClient(host, None, delegatedDN=DN, delegatedGroup=group)

        result = yield self.threadTask(client.getLogTail, system, name)
        gLogger.debug(result)

        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return

        result = result["Value"]

        key = system + "_" + name
        if key not in result:
            self.finish({"success": "false", "error": "%s key is absent in service response" % key})
            return

        log = result[key]

        self.finish({"success": "true", "result": log.replace("\n", "<br>")})

    @asyncGen
    def web_hostAction(self):
        """
        Restart all DIRAC components on a given host
        """

        if "host" not in self.request.arguments:
            self.finish({"success": "false", "error": "No hostname defined"})
            return

        if "action" not in self.request.arguments:
            self.finish({"success": "false", "error": "No action defined"})
            return

        action = self.get_argument("action")
        hosts = self.get_argument("host").split(",")
        version = self.get_argument("version")

        DN = self.getUserDN()
        group = self.getUserGroup()

        actionSuccess = list()
        actionFailed = list()

        for i in hosts:
            client = SystemAdministratorClient(str(i), None, delegatedDN=DN, delegatedGroup=group)
            if action == "restart":
                result = yield self.threadTask(client.restartComponent, str("*"), str("*"))
            elif action == "revert":
                result = yield self.threadTask(client.revertSoftware)
            elif action == "update":
                result = yield self.threadTask(client.updateSoftware, version, timeout=600)
            else:
                error = i + ": Action %s is not defined" % action
                actionFailed.append(error)
                continue

            gLogger.always(result)

            if not result["OK"]:
                if result["Message"].find("Unexpected EOF") > 0:
                    msg = "Signal 'Unexpected EOF' received: %s. Most likely DIRAC components" % result["Message"]
                    msg = i + ": " + msg + " were successfully restarted."
                    actionSuccess.append(msg)
                    continue
                error = i + ": " + result["Message"]
                actionFailed.append(error)
                gLogger.error(error)
            else:
                gLogger.info(result["Value"])
                actionSuccess.append(i)

        self.finish(self.aftermath(actionSuccess, actionFailed, action, "Host"))

    @asyncGen
    def web_componentAction(self):
        """
        Actions which should be done on components. The only parameters is an action
        to perform.
        Returns standard JSON response structure with with service response
        or error messages
        """
        DN = self.getUserDN()
        group = self.getUserGroup()

        if not (("action" in self.request.arguments) and (len(self.get_argument("action")) > 0)):
            self.finish({"success": "false", "error": "No action defined"})
            return

        action = self.get_argument("action")

        if action not in ["restart", "start", "stop"]:
            error = "The request parameters action '%s' is unknown" % action
            gLogger.debug(error)
            self.finish({"success": "false", "error": error})
            return

        result = dict()
        for i in self.request.arguments:
            if i == "action":
                continue

            target = i.split("@")
            if not len(target) == 2:
                continue

            system = self.get_argument(i)
            gLogger.always("System: %s" % system)
            host = target[1]
            gLogger.always("Host: %s" % host)
            component = target[0]
            gLogger.always("Component: %s" % component)
            if host not in result:
                result[host] = list()
            result[host].append([system, component])

        if not result:
            error = "Failed to get component(s) for %s" % action
            gLogger.debug(error)
            self.finish({"success": "false", "error": error})

        gLogger.always(result)
        actionSuccess = list()
        actionFailed = list()

        for hostname in result:

            if not result[hostname]:
                continue

            client = SystemAdministratorClient(hostname, None, delegatedDN=DN, delegatedGroup=group)

            for i in result[hostname]:

                system = i[0]
                component = i[1]

                try:
                    if action == "restart":
                        result = yield self.threadTask(client.restartComponent, system, component)
                    elif action == "start":
                        result = yield self.threadTask(client.startComponent, system, component)
                    elif action == "stop":
                        result = yield self.threadTask(client.stopComponent, system, component)
                    else:
                        result = dict(OK=False)
                        result["Message"] = "Action %s is not valid" % action
                except Exception as x:
                    result = dict(OK=False)
                    result["Message"] = "Exception: %s" % str(x)
                gLogger.debug("Result: %s" % result)

                if not result["OK"]:
                    error = hostname + ": " + result["Message"]
                    actionFailed.append(error)
                    gLogger.error("Failure during component %s: %s" % (action, error))
                else:
                    gLogger.always("Successfully %s component %s" % (action, component))
                    actionSuccess.append(component)

        self.finish(self.aftermath(actionSuccess, actionFailed, action, "Component"))

    def aftermath(self, actionSuccess, actionFailed, action, prefix):

        success = ", ".join(actionSuccess)
        failure = "\n".join(actionFailed)

        if len(actionSuccess) > 1:
            sText = prefix + "s"
        else:
            sText = prefix

        if len(actionFailed) > 1:
            fText = prefix + "s"
        else:
            fText = prefix

        if success and failure:
            sMessage = "%s %sed successfully: %s" % (sText, action, success)
            fMessage = "Failed to %s %s:\n%s" % (action, fText, failure)
            result = sMessage + "\n\n" + fMessage
            return {"success": "true", "result": result}
        elif success and len(failure) < 1:
            result = "%s %sed successfully: %s" % (sText, action, success)
            return {"success": "true", "result": result}
        elif len(success) < 1 and failure:
            result = "Failed to %s %s:\n%s" % (action, fText, failure)
            gLogger.always(result)
            return {"success": "false", "error": result}

        result = "No action has performed due technical failure. Check the logs please"
        gLogger.debug(result)
        return {"success": "false", "error": result}

    def web_getUsersGroups(self):

        result = gConfig.getSections("/Registry/Users")
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        result = result["Value"]

        users = [[x] for x in result]

        result = gConfig.getSections("/Registry/Groups")
        if not result["OK"]:
            self.finish({"success": "false", "error": result["Message"]})
            return
        result = result["Value"]

        groups = [[x] for x in result]

        self.finish({"success": "true", "users": users, "groups": groups, "email": self.getUserEmail()})

    def getUserEmail(self):

        user = self.getUserName()

        if not user:
            gLogger.debug("user value is empty")
            return None

        if user == "anonymous":
            gLogger.debug("user is anonymous")
            return None

        email = gConfig.getValue("/Registry/Users/%s/Email" % user, "")
        gLogger.debug("/Registry/Users/%s/Email - '%s'" % (user, email))
        email = email.strip()

        if not email:
            return None
        return email

    def web_sendMessage(self):
        """
        Send message(not implemented yet) or email getting parameters from request
        """

        email = self.getUserEmail()

        if "subject" not in self.request.arguments:
            result = "subject parameter is not in request... aborting"
            gLogger.debug(result)
            self.finish({"success": "false", "error": result})
            return

        subject = self.checkUnicode(self.get_argument("subject"))
        if not subject:
            subject = "Message from %s" % email

        if "message" not in self.request.arguments:
            result = "msg parameter is not in request... aborting"
            gLogger.debug(result)
            self.finish({"success": "false", "error": result})
            return

        body = self.checkUnicode(self.get_argument("message"))
        if not len(body) > 0:
            result = "Message body has zero length... aborting"
            gLogger.debug(result)
            self.finish({"success": "false", "error": result})
            return

        users = self.get_argument("users").split(",")

        groups = self.get_argument("groups").split(",")

        gLogger.info("List of groups from request: %s" % groups)
        if groups:
            for g in groups:
                userList = self.getUsersFromGroup(g)
                gLogger.info("Get users: %s from group %s" % (userList, g))
                if userList:
                    users.extend(userList)

        gLogger.info("Merged list of users from users and group %s" % users)

        if not len(users) > 0:
            error = "Length of list of recipients is zero size"
            gLogger.info(error)
            self.finish({"success": "false", "error": error})
            return

        users = uniqueElements(users)
        gLogger.info("Final list of users to send message/mail: %s" % users)

        sendDict = self.getMailDict(users)
        self.finish(self.sendMail(sendDict, subject, body, email))

    def checkUnicode(self, text=None):
        """
        Check if value is unicode or not and return properly converted string
        Arguments are string and unicode/string, return value is a string
        """
        try:
            text = text.decode("utf-8", "replace")
        except Exception:
            pass
        text = text.encode("utf-8")
        gLogger.debug(text)

        return text

    def getUsersFromGroup(self, groupname=None):

        if not groupname:
            gLogger.debug("Argument groupname is missing")
            return None

        users = gConfig.getValue("/Registry/Groups/%s/Users" % groupname, [])
        gLogger.debug("%s users: %s" % (groupname, users))
        if not users:
            gLogger.debug("No users for group %s found" % groupname)
            return None
        return users

    def getMailDict(self, names=None):
        """
        Convert list of usernames to dict like { e-mail : full name }
        Argument is a list. Return value is a dict
        """

        resultDict = dict()
        if not names:
            return resultDict

        for user in names:
            email = gConfig.getValue("/Registry/Users/%s/Email" % user, "")
            gLogger.debug("/Registry/Users/%s/Email - '%s'" % (user, email))
            email = email.strip()

            if not email:
                gLogger.error("Can't find value for option /Registry/Users/%s/Email" % user)
                continue

            fname = gConfig.getValue("/Registry/Users/%s/FullName" % user, "")
            gLogger.debug("/Registry/Users/%s/FullName - '%s'" % (user, fname))
            fname = fname.strip()

            if not fname:
                fname = user
                gLogger.debug("FullName is absent, name to be used: %s" % fname)

            resultDict[email] = fname

        return resultDict

    def sendMail(self, sendDict=None, title=None, body=None, fromAddress=None):
        """
        Sending an email using sendDict: { e-mail : name } as addressbook
        title and body is the e-mail's Subject and Body
        fromAddress is an email address in behalf of whom the message is sent
        Return success/failure JSON structure
        """

        if not sendDict:
            result = ""
            gLogger.debug(result)
            return {"success": "false", "error": result}

        if not title:
            result = "title argument is missing"
            gLogger.debug(result)
            return {"success": "false", "error": result}

        if not body:
            result = "body argument is missing"
            gLogger.debug(result)
            return {"success": "false", "error": result}

        if not fromAddress:
            result = "fromAddress argument is missing"
            gLogger.debug(result)
            return {"success": "false", "error": result}

        sentSuccess = list()
        sentFailed = list()
        gLogger.debug("Initializing Notification client")
        ntc = NotificationClient()

        for email, name in sendDict.iteritems():
            result = ntc.sendMail(email, title, body, fromAddress, False)
            if not result["OK"]:
                error = name + ": " + result["Message"]
                sentFailed.append(error)
                gLogger.error("Sent failure: ", error)
            else:
                gLogger.info("Successfully sent to %s" % name)
                sentSuccess.append(name)

        success = ", ".join(sentSuccess)
        failure = "\n".join(sentFailed)

        if success and failure:
            result = "Successfully sent e-mail to: "
            result = result + success + "\n\nFailed to send e-mail to:\n" + failure
            gLogger.debug(result)
            return {"success": "true", "result": result}
        elif success and len(failure) < 1:
            result = "Successfully sent e-mail to: %s" % success
            gLogger.debug(result)
            return {"success": "true", "result": result}
        elif len(success) < 1 and failure:
            result = "Failed to sent email to:\n%s" % failure
            gLogger.debug(result)
            return {"success": "false", "error": result}

        result = "No messages were sent due technical failure"
        gLogger.debug(result)
        return {"success": "false", "error": result}

    @asyncGen
    def web_getComponentNames(self):

        result = None

        setup = self.getUserSetup().split("-")[-1]
        systemList = []
        componentTypes = ["Services", "Agents"]
        componentTypes = self.get_arguments("ComponentType") or componentTypes

        retVal = gConfig.getSections("/Systems")

        if retVal["OK"]:
            systems = retVal["Value"]
            for i in systems:
                for compType in componentTypes:
                    compPath = "/Systems/%s/%s/%s" % (i, setup, compType)
                    retVal = gConfig.getSections(compPath)
                    if retVal["OK"]:
                        components = retVal["Value"]
                        systemList += [{"Name": j} for j in components]

            result = {"success": "true", "result": systemList}
        else:
            result = {"success": "false", "error": result["Message"]}

        self.finish(result)

    @asyncGen
    def web_getSelectionData(self):

        data = {"Hosts": []}

        setup = self.getUserSetup().split("-")[-1]

        hosts = []
        result = ComponentMonitoringClient().getHosts({}, False, False)
        if result["OK"]:
            for hostDict in result["Value"]:
                if hostDict.get("HostName"):
                    data["Hosts"].append([hostDict["HostName"]])

        componentTypes = ["Services", "Agents", "Executors"]
        componentTypes = self.get_arguments("ComponentType") or componentTypes

        retVal = gConfig.getSections("/Systems")

        components = []
        componentNames = []
        data["ComponentModule"] = []
        data["ComponentName"] = []
        if retVal["OK"]:
            systems = retVal["Value"]
            for i in systems:
                for compType in componentTypes:
                    compPath = "/Systems/%s/%s/%s" % (i, setup, compType)
                    retVal = gConfig.getSections(compPath)
                    if retVal["OK"]:
                        records = retVal["Value"]
                        componentNames += [[cnames] for cnames in records]
                        for record in records:
                            modulepath = "%s/%s/Module" % (compPath, record)
                            module = gConfig.getValue(modulepath, "")
                            if module != "" and module not in components:
                                components += [module]
                                data["ComponentModule"].append([module])
                            elif record not in components and module == "":
                                data["ComponentModule"].append([record])
                                components += [record]

            data["ComponentName"] = componentNames
            data["ComponentName"].sort()
            data["ComponentModule"].sort()

        else:
            data = {"success": "false", "error": result["Message"]}

        self.finish(data)

    @asyncGen
    def web_ComponentLocation(self):

        _setup = self.getUserSetup()
        setup = _setup.split("-")[-1]

        hosts = []
        result = ComponentMonitoringClient().getHosts({}, False, False)
        if result["OK"]:
            hosts = [h["HostName"] for h in result["Value"] if h.get("HostName")]

        componentTypes = ["Services", "Agents"]
        componentTypes = self.get_arguments("ComponentType") or componentTypes

        componentNames = []
        if "ComponentName" in self.request.arguments:
            componentNames = list(json.loads(self.get_argument("ComponentName")))

        componentModules = []
        if "ComponentModule" in self.request.arguments:
            componentModules = list(json.loads(self.get_argument("ComponentModule")))

        showAll = 0
        if "showAll" in self.request.arguments:
            showAll = int(self.get_argument("showAll"))

        selectedHosts = []
        if "Hosts" in self.request.arguments:  # we only use the selected host(s)
            selectedHosts = list(json.loads(self.get_argument("Hosts")))
        retVal = gConfig.getSections("/Systems")

        compMatching = {}
        fullNames = []
        if retVal["OK"]:
            systems = retVal["Value"]
            for i in systems:
                for compType in componentTypes:
                    compPath = "/Systems/%s/%s/%s" % (i, setup, compType)
                    retVal = gConfig.getSections(compPath)
                    if retVal["OK"]:
                        components = retVal["Value"]
                        for j in components:
                            path = "%s/%s" % (i, j)
                            if j in componentNames:
                                fullNames += [path]
                                compMatching[path] = path
                            modulepath = "%s/%s/Module" % (compPath, j)
                            module = gConfig.getValue(modulepath, "")
                            if module != "" and module in componentModules:
                                fullNames += [path]
                            elif module == "" and j in componentModules:
                                fullNames += [path]

                            compMatching[path] = module if module != "" else path

        records = []
        if fullNames:
            condDict = {"Setup": _setup, "ComponentName": fullNames}
        else:
            if len(componentTypes) < 2:
                cType = "agent" if componentTypes[-1] == "Agents" else "service"
                condDict = {"Setup": _setup, "Type": cType}
            else:
                condDict = {"Setup": _setup}

        gLogger.debug("condDict" + str(condDict))
        retVal = gMonitor.getComponentsStatus(condDict)

        today = datetime.datetime.today()
        if retVal["OK"]:
            components = retVal["Value"][0]
            for setup in components:
                for cType in components[setup]:
                    for name in components[setup][cType]:
                        for component in components[setup][cType][name]:
                            if selectedHosts and "Host" in component and component["Host"] not in selectedHosts:
                                continue
                            elif "Host" in component and component["Host"] not in hosts:
                                continue
                            if "LastHeartbeat" in component:
                                dateDiff = today - component["LastHeartbeat"]
                            else:
                                dateDiff = today - today

                            if showAll == 0 and dateDiff.days >= 2 and "Host" in component:
                                continue

                            for conv in component:
                                component[conv] = str(component[conv])
                            component["ComponentModule"] = (
                                compMatching[component["ComponentName"]]
                                if component["ComponentName"] in compMatching
                                else component["ComponentName"]
                            )
                            records += [component]

            result = {"success": "true", "result": records}
        else:
            result = {"success": "false", "error": retVal["Message"]}

        self.finish(result)



import json

from DIRAC import gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.RequestManagementSystem.Client.ReqClient import ReqClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class RequestMonitorHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getRequestMonitorData(self):
        callback = {}
        req = self.__request()

        result = yield self.threadTask(
            ReqClient().getRequestSummaryWeb, req, self.globalSort, self.pageNumber, self.numberOfJobs
        )

        if not result["OK"]:
            self.finish({"success": "false", "result": [], "total": 0, "error": result["Message"]})
            return

        result = result["Value"]

        if "TotalRecords" not in result:
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (result["TotalRecords"] > 0):
            self.finish(
                {"success": "false", "result": [], "total": 0, "error": "There were no data matching your selection"}
            )
            return

        if not ("ParameterNames" in result and "Records" in result):
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (len(result["ParameterNames"]) > 0):
            self.finish({"success": "false", "result": [], "total": -1, "error": "ParameterNames field is missing"})
            return

        if not (len(result["Records"]) > 0):
            self.finish({"success": "false", "result": [], "total": 0, "Message": "There are no data to display"})
            return

        callback = []
        jobs = result["Records"]
        head = result["ParameterNames"]
        headLength = len(head)

        jobs = result["Records"]
        head = result["ParameterNames"]
        headLength = len(head)
        for i in jobs:
            tmp = {}
            for j in range(0, headLength):
                if j == 2:
                    if i[j] == "None":
                        i[j] = "-"
                tmp[head[j]] = i[j]
            callback.append(tmp)
        total = result["TotalRecords"]
        total = result["TotalRecords"]
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        if "Extras" in result:
            st = self.__dict2string({})
            extra = result["Extras"]
            callback = {
                "success": "true",
                "result": callback,
                "total": total,
                "extra": extra,
                "request": st,
                "date": timestamp,
            }
        else:
            callback = {"success": "true", "result": callback, "total": total, "date": timestamp}
        self.finish(callback)

    def __dict2string(self, req):
        result = ""
        try:
            for key, value in req.iteritems():
                result = result + str(key) + ": " + ", ".join(value) + "; "
        except Exception as x:
            pass
            gLogger.info("\033[0;31m Exception: \033[0m %s" % x)
        result = result.strip()
        result = result[:-1]
        return result

    @asyncGen
    def web_getSelectionData(self):
        callback = {}
        group = self.getUserGroup()
        user = self.getUserName()
        if user == "Anonymous":
            self.finish({"success": "false", "result": [], "total": 0, "error": "Insufficient rights"})
        else:
            # R E Q U E S T T Y P E
            result = yield self.threadTask(ReqClient().getDistinctValuesWeb, "Type")
            if result["OK"]:
                reqtype = list()
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        reqtype.append([str(i)])
                else:
                    reqtype = [["Nothing to display"]]
            else:
                reqtype = [["Error during RPC call"]]
            callback["operationType"] = reqtype
            # U S E R
            result = yield self.threadTask(ReqClient().getDistinctValuesWeb, "OwnerDN")

            if result["OK"]:
                owner = []
                for dn in result["Value"]:

                    owner.append([dn])
                if len(owner) < 2:
                    owner = [["Nothing to display"]]
            else:
                owner = [["Error during RPC call"]]
            callback["owner"] = owner
            # G R O U P
            result = yield self.threadTask(ReqClient().getDistinctValuesWeb, "OwnerGroup")
            gLogger.info("getDistinctValuesWeb(OwnerGroup)", result)
            if result["OK"]:
                ownerGroup = list()
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        ownerGroup.append([str(i)])
                else:
                    ownerGroup = [["Nothing to display"]]
            else:
                ownerGroup = [["Error during RPC call"]]
            callback["ownerGroup"] = ownerGroup
            # S T A T U S
            result = yield self.threadTask(ReqClient().getDistinctValuesWeb, "Status")

            if result["OK"]:
                status = list()
                if len(result["Value"]) > 0:
                    for i in result["Value"]:
                        status.append([str(i)])
                else:
                    status = [["Nothing to display"]]
            else:
                status = [["Error during RPC call"]]
            callback["status"] = status
            self.finish(callback)

    ################################################################################
    def __request(self):
        self.numberOfJobs = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))
        self.globalSort = [["JobID", "DESC"]]
        group = self.getUserGroup()
        user = self.getUserName()
        req = {}
        found = False

        jobids = list(json.loads(self.get_argument("id", "[]")))
        if jobids:
            req["JobID"] = jobids
            found = True

        reqids = list(json.loads(self.get_argument("reqId", "[]")))
        if reqids and not found:
            req["RequestID"] = reqids
            found = True

        if not found:
            value = list(json.loads(self.get_argument("operationType", "[]")))
            if value:
                req["Type"] = value

            value = list(json.loads(self.get_argument("ownerGroup", "[]")))
            if value:
                req["OwnerGroup"] = value

            value = list(json.loads(self.get_argument("status", "[]")))
            if value:
                req["Status"] = value

            value = list(json.loads(self.get_argument("owner", "[]")))
            if value:
                req["OwnerDN"] = value

            sort = json.loads(self.get_argument("sort", "[]"))
            if sort:
                self.globalSort = [[i["property"], i["direction"]] for i in sort]

        if self.get_argument("startDate", ""):
            req["FromDate"] = self.get_argument("startDate")
            if self.get_argument("startTime", ""):
                req["FromDate"] += " " + self.get_argument("startTime")

        if self.get_argument("endDate", ""):
            req["ToDate"] = self.get_argument("endDate")
            if self.get_argument("endTime", ""):
                req["ToDate"] += " " + self.get_argument("endTime")

        if self.get_argument("date", ""):
            req["LastUpdate"] = self.get_argument("date")
        gLogger.info("REQUEST:", req)
        return req



import base64
import zlib
import json

from DIRAC import S_OK
from DIRAC.Core.Utilities import DEncode
from DIRAC.Core.DISET.ThreadConfig import ThreadConfig
from DIRAC.FrameworkSystem.Client.UserProfileClient import UserProfileClient

from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler, WErr, authorization


class UPHandler(WebHandler):
    DEFAULT_AUTHORIZATION = "authenticated"
    __tc = ThreadConfig()

    def initializeRequest(self):
        self.set_header("Pragma", "no-cache")
        self.set_header("Cache-Control", "max-age=0, no-store, no-cache, must-revalidate")
        # Do not use the defined user setup. Use the web one to show the same profile independently of user setup
        self.__tc.setSetup(False)

    def web_saveAppState(self, obj, app, name, state):
        """Save App State

        :param str obj: object
        :param str app: application
        :param str name: name
        :param str state: state

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        data = base64.b64encode(zlib.compress(DEncode.encode(state), 9))
        # before we save the state (modify the state) we have to remember the actual access: ReadAccess and PublishAccess
        result = up.getVarPermissions(name)
        access = {
            "ReadAccess": "USER",
            "PublishAccess": "USER",
        }  # this is when the application/desktop does not exists.
        if result["OK"]:
            access = result["Value"]

        result = up.storeVar(name, data)
        if result["OK"]:
            # change the access to the application/desktop
            result = up.setVarPermissions(name, access)
        return S_OK() if result["OK"] else result

    def web_makePublicAppState(self, obj, app, name, access="ALL"):
        """Make public application state

        :param str obj: object
        :param str app: application
        :param str name: name
        :param str access: access type

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        access = access.upper()
        if access not in ("ALL", "VO", "GROUP", "USER"):
            raise WErr(400, "Invalid access")

        revokeAccess = {"ReadAccess": access}
        if access == "USER":  # if we make private a state,
            # we have to revoke from the public as well
            revokeAccess["PublishAccess"] = "USER"

        # TODO: Check access is in either 'ALL', 'VO' or 'GROUP'
        return up.setVarPermissions(name, revokeAccess)

    def web_loadAppState(self, obj, app, name):
        """Load application state

        :param str obj: object
        :param str app: application
        :param str name: name

        :return: dict
        """
        result = UserProfileClient("Web/%s/%s" % (obj, app)).retrieveVar(name)
        if not result["OK"]:
            return result
        data = result["Value"]
        return DEncode.decode(zlib.decompress(base64.b64decode(data)))[0]

    def web_loadUserAppState(self, obj, app, user, group, name):
        """Load user application state

        :param str obj: object
        :param str app: application
        :param str user: user name
        :param str group: group name
        :param str name: name

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        result = up.retrieveVarFromUser(user, group, name)
        if not result["OK"]:
            return result
        data = result["Value"]
        return DEncode.decode(zlib.decompress(base64.b64decode(data)))[0]

    @authorization(["all"])
    def web_listAppState(self, obj, app):
        """Get list application state

        :param str obj: object
        :param str app: application

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        result = up.retrieveAllVars()
        if not result["OK"]:
            return result
        data = result["Value"]
        # Unpack data
        return {k: json.loads(DEncode.decode(zlib.decompress(base64.b64decode(data[k])))[0]) for k in data}

    def web_delAppState(self, obj, app, name):
        """Delete application state

        :param str obj: object
        :param str app: application
        :param str name: name

        :return: dict
        """
        return UserProfileClient("Web/%s/%s" % (obj, app)).deleteVar(name)

    @authorization(["all"])
    def web_listPublicDesktopStates(self, obj, app):
        """Get list public desktop states

        :param str obj: object
        :param str app: application

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        result = up.listAvailableVars()
        if not result["OK"]:
            return result
        data = result["Value"]
        paramNames = ["UserName", "Group", "VO", "desktop"]

        records = [dict(zip(paramNames, i)) for i in data]
        sharedDesktops = {}
        for i in records:
            result = up.getVarPermissions(i["desktop"])
            if not result["OK"]:
                return result
            if result["Value"]["ReadAccess"] == "ALL":
                print(i["UserName"], i["Group"], i)
                result = up.retrieveVarFromUser(i["UserName"], i["Group"], i["desktop"])
                if not result["OK"]:
                    return result
                if i["UserName"] not in sharedDesktops:
                    sharedDesktops[i["UserName"]] = {}
                sharedDesktops[i["UserName"]][i["desktop"]] = json.loads(
                    DEncode.decode(zlib.decompress(base64.b64decode(result["Value"])))[0]
                )
                sharedDesktops[i["UserName"]]["Metadata"] = i
        return sharedDesktops

    def web_makePublicDesktopState(self, obj, app, name, access="ALL"):
        """Make public desktop state

        :param str obj: object
        :param str app: application
        :param str name: name
        :param str access: access type

        :return: dict
        """
        up = UserProfileClient("Web/application/desktop")
        access = access.upper()
        if access not in ("ALL", "VO", "GROUP", "USER"):
            raise WErr(400, "Invalid access")
        # TODO: Check access is in either 'ALL', 'VO' or 'GROUP'
        return up.setVarPermissions(name, {"ReadAccess": access})

    def web_changeView(self, obj, app, desktop, view):
        """Change view

        :param str obj: object
        :param str app: application
        :param str desktop: desktop name
        :param str view: view

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        result = up.retrieveVar(desktop)
        if not result["OK"]:
            return result
        data = result["Value"]
        oDesktop = json.loads(DEncode.decode(zlib.decompress(base64.b64decode(data)))[0])
        oDesktop[str("view")] = str(view)
        oDesktop = json.dumps(oDesktop)
        data = base64.b64encode(zlib.compress(DEncode.encode(oDesktop), 9))
        return up.storeVar(desktop, data)

    @authorization(["all"])
    def web_listPublicStates(self, obj, app):
        """Get list public state

        :param str obj: object
        :param str app: application

        :return: dict
        """
        user = self.getUserName()

        up = UserProfileClient("Web/%s/%s" % (obj, app))
        retVal = up.getUserProfileNames({"PublishAccess": "ALL"})
        if not retVal["OK"]:
            raise WErr.fromSERROR(retVal)
        records = retVal["Value"]
        if not records:
            raise WErr(404, "There are no public states!")

        mydesktops = {"name": "My Desktops", "group": "", "vo": "", "user": "", "iconCls": "my-desktop", "children": []}
        shareddesktops = {
            "name": "Shared Desktops",
            "group": "",
            "vo": "",
            "user": "",
            "expanded": "true",
            "iconCls": "shared-desktop",
            "children": [],
        }

        myapplications = {"name": "My Applications", "group": "", "vo": "", "user": "", "children": []}
        sharedapplications = {
            "name": "Shared Applications",
            "group": "",
            "vo": "",
            "user": "",
            "expanded": "true",
            "iconCls": "shared-desktop",
            "children": [],
        }

        desktopsApplications = {
            "text": ".",
            "children": [
                {"name": "Desktops", "group": "", "vo": "", "user": "", "children": [mydesktops, shareddesktops]},
                {
                    "name": "Applications",
                    "group": "",
                    "vo": "",
                    "user": "",
                    "children": [myapplications, sharedapplications],
                },
            ],
        }
        for record in records:
            permissions = record["permissions"]
            if permissions["PublishAccess"] == "ALL":
                if record["app"] == "desktop":
                    record["type"] = "desktop"
                    record["leaf"] = "true"
                    record["iconCls"] = ("core-desktop-icon",)
                    if record["user"] == user:
                        mydesktops["children"].append(record)
                    else:
                        shareddesktops["children"].append(record)
                else:
                    record["type"] = "application"
                    record["leaf"] = "true"
                    record["iconCls"] = "core-application-icon"
                    if record["user"] == user:
                        myapplications["children"].append(record)
                    else:
                        sharedapplications["children"].append(record)

        return desktopsApplications

    def web_publishAppState(self, obj, app, name, access="ALL"):
        """Publish application state

        :param str obj: object
        :param str app: application
        :param str name: name
        :param str access: access type

        :return: dict
        """
        up = UserProfileClient("Web/%s/%s" % (obj, app))
        access = access.upper()
        if access not in ("ALL", "VO", "GROUP", "USER"):
            raise WErr(400, "Invalid access")

        return up.setVarPermissions(name, {"PublishAccess": access, "ReadAccess": access})



from WebAppDIRAC.Lib.WebHandler import _WebHandler as WebHandler
from DIRAC.Core.Utilities import Time


class ExampleAppHandler(WebHandler):

    DEFAULT_AUTHORIZATION = "all"

    def web_getJobData(self):
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        total = 5
        values = [
            {"ExampleId": 1, "ExampleValue": "Zoli"},
            {"ExampleId": 2, "ExampleValue": "a"},
            {"ExampleId": 3, "ExampleValue": "aaaa"},
            {"ExampleId": 4, "ExampleValue": "bbbb"},
            {"ExampleId": 5, "ExampleValue": "adsd"},
        ]
        return {"success": "true", "result": values, "total": total, "date": timestamp}

    def web_getSelectionData(self):
        return {"firstName": ["A", "C", "D"], "lastName": ["wwww", "dsds", "sads"]}



import json
from time import time

from DIRAC import gLogger, gConfig
from DIRAC.WorkloadManagementSystem.Client.WMSAdministratorClient import WMSAdministratorClient

from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class JobSummaryHandler(WebHandler):

    AUTH_PROPS = "all"

    pageNumber = 0
    numberOfJobs = 25
    globalSort = [["GridSite", "ASC"]]

    @asyncGen
    def web_getSelectionData(self):
        callback = {}

        result = yield self.threadTask(WMSAdministratorClient().getSiteSummarySelectors)
        gLogger.info("\033[0;31m ++++++: \033[0m %s" % result)
        if result["OK"]:
            result = result["Value"]
            if len(result.get("Status", [])) > 0:
                status = [[str("All")]]
                for i in result["Status"]:
                    status.append([str(i)])
            else:
                status = [["Nothing to display"]]
            callback["status"] = status
            if len(result.get("GridType", [])) > 0:
                gridtype = [[str("All")]]
                for i in result["GridType"]:
                    gridtype.append([str(i)])
            else:
                gridtype = [["Nothing to display"]]
            callback["gridtype"] = gridtype
            if len(result.get("MaskStatus", [])) > 0:
                maskstatus = [[str("All")]]
                for i in result["MaskStatus"]:
                    maskstatus.append([str(i)])
            else:
                maskstatus = [["Nothing to display"]]
            callback["maskstatus"] = maskstatus
            if len(result.get("Site", [])) > 0:
                s = list(result["Site"])
                tier1 = gConfig.getValue("/WebApp/PreferredSites", [])
                site = [["All"]]
                for i in tier1:
                    site.append([str(i)])
                for i in s:
                    if i not in tier1:
                        site.append([str(i)])
            else:
                site = [["Error during RPC call"]]
            callback["site"] = site
            if len(result.get("Country", [])) > 0:
                country = [["All"]]
                countryCode = self.__getCountries()
                for i in result["Country"]:
                    if i in countryCode:
                        j = countryCode[i]
                        country.append([str(j)])
            else:
                country = [["Nothing to display"]]
            country.sort()
            callback["country"] = country
        else:
            callback["status"] = [["Error during RPC call"]]
            callback["gridtype"] = [["Error during RPC call"]]
            callback["maskstatus"] = [["Error during RPC call"]]
            callback["site"] = [["Error during RPC call"]]
            callback["country"] = [["Error during RPC call"]]
        # ##
        self.finish(callback)

    @asyncGen
    def web_getData(self):
        pagestart = time()
        result = self.__request()
        gLogger.always("getSiteSummaryWeb(%s,%s,%s,%s)" % (result, self.globalSort, self.pageNumber, self.numberOfJobs))
        retVal = yield self.threadTask(
            WMSAdministratorClient().getSiteSummaryWeb, result, [], self.pageNumber, self.numberOfJobs
        )
        gLogger.always("\033[0;31m YO: \033[0m", result)
        if retVal["OK"]:
            if retVal["Value"].get("TotalRecords", 0) > 0:
                if "ParameterNames" in retVal["Value"] and "Records" in retVal["Value"]:
                    if len(retVal["Value"].get("ParameterNames", [])) > 0:
                        if len(retVal["Value"].get("Records", [])) > 0:
                            result = []
                            jobs = retVal["Value"]["Records"]
                            head = retVal["Value"]["ParameterNames"]
                            headLength = len(head)
                            countryCode = self.__getCountries()
                            for i in jobs:
                                tmp = {head[j]: i[j] for j in range(headLength)}
                                tmp["FullCountry"] = countryCode[i[2]] if i[2] in countryCode else "Unknown"
                                result.append(tmp)
                            total = retVal["Value"]["TotalRecords"]
                            if "Extras" in retVal["Value"]:
                                extra = retVal["Value"]["Extras"]
                                result = {"success": "true", "result": result, "total": total, "extra": extra}
                            else:
                                result = {"success": "true", "result": result, "total": total}
                        else:
                            result = {"success": "false", "result": "", "error": "There are no data to display"}
                    else:
                        result = {"success": "false", "result": "", "error": "ParameterNames field is undefined"}
                else:
                    result = {"success": "false", "result": "", "error": "Data structure is corrupted"}
            else:
                result = {"success": "false", "result": "", "error": "There were no data matching your selection"}
        else:
            gLogger.always("- E R R O R -")
            result = {"success": "false", "error": retVal["Message"]}
        gLogger.info("\033[0;31m SITESUMMARY INDEX REQUEST: \033[0m %s" % (time() - pagestart))
        self.finish(result)

    def __request(self):
        req = {}

        if "id" in self.request.arguments:
            jobids = list(json.loads(self.get_argument("id")))
            if jobids:
                req["JobID"] = jobids

        elif len(self.get_argument("expand", [])) > 0:
            self.globalSort = [["GridSite", "ASC"]]
            self.numberOfJobs = 500
            self.pageNumber = 0
            req["ExpandSite"] = self.get_argument("expand", "")
        else:
            self.pageNumber = 0
            self.numberOfJobs = 500
            countries = list(json.loads(self.get_argument("country", "[]")))
            if countries:
                code = self.__getCountriesReversed()
                newValue = [code[i] for i in countries if i in code]
                req["Country"] = newValue

            site = list(json.loads(self.get_argument("site", "[]")))
            if site:
                req["Site"] = site

            status = list(json.loads(self.get_argument("status", "[]")))
            if status:
                req["Status"] = status

            maskstatus = list(json.loads(self.get_argument("maskstatus", "[]")))
            if maskstatus:
                req["MaskStatus"] = maskstatus

            gridtype = list(json.loads(self.get_argument("gridtype", "[]")))
            owner = list(json.loads(self.get_argument("owner", "[]")))
            if gridtype:
                req["GridType"] = gridtype
            elif owner:
                req["Owner"] = owner

            if self.get_argument("date", ""):
                req["LastUpdate"] = self.get_argument("date")

        gLogger.info("REQUEST:", req)
        return req

    def __getCountriesReversed(self):
        """
        Return the dictionary of country names and
        corresponding country code top-level domain (ccTLD)
        """
        result = self.__getCountries()
        return dict(zip(result.values(), result))

    @staticmethod
    def __getCountries():
        """
        Return the dictionary of country code top-level domain (ccTLD) and
        corresponding country name
        """
        return {
            "af": "Afghanistan",
            "al": "Albania",
            "dz": "Algeria",
            "as": "American Samoa",
            "ad": "Andorra",
            "ao": "Angola",
            "ai": "Anguilla",
            "aq": "Antarctica",
            "ag": "Antigua and Barbuda",
            "ar": "Argentina",
            "am": "Armenia",
            "aw": "Aruba",
            "au": "Australia",
            "at": "Austria",
            "az": "Azerbaijan",
            "bs": "Bahamas",
            "bh": "Bahrain",
            "bd": "Bangladesh",
            "bb": "Barbados",
            "by": "Belarus",
            "be": "Belgium",
            "bz": "Belize",
            "bj": "Benin",
            "bm": "Bermuda",
            "bt": "Bhutan",
            "bo": "Bolivia",
            "ba": "Bosnia and Herzegowina",
            "bw": "Botswana",
            "bv": "Bouvet Island",
            "br": "Brazil",
            "io": "British Indian Ocean Territory",
            "bn": "Brunei Darussalam",
            "bg": "Bulgaria",
            "bf": "Burkina Faso",
            "bi": "Burundi",
            "kh": "Cambodia",
            "cm": "Cameroon",
            "ca": "Canada",
            "cv": "Cape Verde",
            "ky": "Cayman Islands",
            "cf": "Central African Republic",
            "td": "Chad",
            "cl": "Chile",
            "cn": "China",
            "cx": "Christmas Island",
            "cc": "Cocos Islands",
            "co": "Colombia",
            "km": "Comoros",
            "cg": "Congo",
            "cd": "Congo",
            "ck": "Cook Islands",
            "cr": "Costa Rica",
            "ci": "Cote D'Ivoire",
            "hr": "Croatia",
            "cu": "Cuba",
            "cy": "Cyprus",
            "cz": "Czech Republic",
            "dk": "Denmark",
            "dj": "Djibouti",
            "dm": "Dominica",
            "do": "Dominican Republic",
            "tp": "East Timor",
            "ec": "Ecuador",
            "eg": "Egypt",
            "sv": "El Salvador",
            "gq": "Equatorial Guinea",
            "er": "Eritrea",
            "ee": "Estonia",
            "et": "Ethiopia",
            "fk": "Falkland Islands",
            "fo": "Faroe Islands",
            "fj": "Fiji",
            "fi": "Finland",
            "fr": "France",
            "fx": "France, metropolitan",
            "gf": "French Guiana",
            "pf": "French Polynesia",
            "tf": "French Southern Territories",
            "ga": "Gabon",
            "gm": "Gambia",
            "ge": "Georgia",
            "de": "Germany",
            "gh": "Ghana",
            "gi": "Gibraltar",
            "gr": "Greece",
            "gl": "Greenland",
            "gd": "Grenada",
            "gp": "Guadeloupe",
            "gu": "Guam",
            "gt": "Guatemala",
            "gn": "Guinea",
            "gw": "Guinea-Bissau",
            "gy": "Guyana",
            "ht": "Haiti",
            "hm": "Heard and Mc Donald Islands",
            "va": "Vatican City",
            "hn": "Honduras",
            "hk": "Hong Kong",
            "hu": "Hungary",
            "is": "Iceland",
            "in": "India",
            "id": "Indonesia",
            "ir": "Iran",
            "iq": "Iraq",
            "ie": "Ireland",
            "il": "Israel",
            "it": "Italy",
            "jm": "Jamaica",
            "jp": "Japan",
            "jo": "Jordan",
            "kz": "Kazakhstan",
            "ke": "Kenya",
            "ki": "Kiribati",
            "kp": "Korea",
            "kr": "Korea",
            "kw": "Kuwait",
            "kg": "Kyrgyzstan",
            "la": "Lao",
            "lv": "Latvia",
            "lb": "Lebanon",
            "ls": "Lesotho",
            "lr": "Liberia",
            "ly": "Libyan",
            "li": "Liechtenstein",
            "lt": "Lithuania",
            "lu": "Luxembourg",
            "mo": "Macau",
            "mk": "Macedonia",
            "mg": "Madagascar",
            "mw": "Malawi",
            "my": "Malaysia",
            "mv": "Maldives",
            "ml": "Mali",
            "mt": "Malta",
            "mh": "Marshall Islands",
            "mq": "Martinique",
            "mr": "Mauritania",
            "mu": "Mauritius",
            "yt": "Mayotte",
            "mx": "Mexico",
            "fm": "Micronesia",
            "md": "Moldova",
            "mc": "Monaco",
            "mn": "Mongolia",
            "ms": "Montserrat",
            "ma": "Morocco",
            "mz": "Mozambique",
            "mm": "Myanmar",
            "na": "Namibia",
            "nr": "Nauru",
            "np": "Nepal",
            "nl": "Netherlands",
            "an": "Netherlands Antilles",
            "nc": "New Caledonia",
            "nz": "New Zealand",
            "ni": "Nicaragua",
            "ne": "Niger",
            "ng": "Nigeria",
            "nu": "Niue",
            "nf": "Norfolk Island",
            "mp": "Northern Mariana Islands",
            "no": "Norway",
            "om": "Oman",
            "pk": "Pakistan",
            "pw": "Palau",
            "pa": "Panama",
            "pg": "Papua New Guinea",
            "py": "Paraguay",
            "pe": "Peru",
            "ph": "Philippines",
            "pn": "Pitcairn",
            "pl": "Poland",
            "pt": "Portugal",
            "pr": "Puerto Rico",
            "qa": "Qatar",
            "re": "Reunion",
            "ro": "Romania",
            "ru": "Russia",
            "rw": "Rwanda",
            "kn": "Saint Kitts and Nevis",
            "lc": "Saint Lucia",
            "vc": "Saint Vincent and the Grenadines",
            "ws": "Samoa",
            "sm": "San Marino",
            "st": "Sao Tome and Principe",
            "sa": "Saudi Arabia",
            "sn": "Senegal",
            "sc": "Seychelles",
            "sl": "Sierra Leone",
            "sg": "Singapore",
            "sk": "Slovakia",
            "si": "Slovenia",
            "sb": "Solomon Islands",
            "so": "Somalia",
            "za": "South Africa",
            "gs": "South Georgia and the South Sandwich Islands",
            "es": "Spain",
            "lk": "Sri Lanka",
            "sh": "St. Helena",
            "pm": "St. Pierre and Miquelon",
            "sd": "Sudan",
            "sr": "Suriname",
            "sj": "Svalbard and Jan Mayen Islands",
            "sz": "Swaziland",
            "se": "Sweden",
            "ch": "Switzerland",
            "sy": "Syrian Arab Republic",
            "tw": "Taiwan",
            "tj": "Tajikistan",
            "tz": "Tanzania",
            "th": "Thailand",
            "tg": "Togo",
            "tk": "Tokelau",
            "to": "Tonga",
            "tt": "Trinidad and Tobago",
            "tn": "Tunisia",
            "tr": "Turkey",
            "tm": "Turkmenistan",
            "tc": "Turks and Caicos Islands",
            "tv": "Tuvalu",
            "ug": "Uganda",
            "ua": "Ukraine",
            "ae": "United Arab Emirates",
            "gb": "United Kingdom",
            "uk": "United Kingdom",
            "us": "United States",
            "um": "United States Minor Outlying Islands",
            "uy": "Uruguay",
            "uz": "Uzbekistan",
            "vu": "Vanuatu",
            "ve": "Venezuela",
            "vn": "Viet Nam",
            "vg": "Virgin Islands (British)",
            "vi": "Virgin Islands (U.S.)",
            "wf": "Wallis and Futuna Islands",
            "eh": "Western Sahara",
            "ye": "Yemen",
            "yu": "Yugoslavia",
            "zm": "Zambia",
            "zw": "Zimbabwe",
            "su": "Soviet Union",
        }



""" WebAppDIRAC
"""

# Define Version
from pkg_resources import get_distribution, DistributionNotFound

try:
    __version__ = get_distribution(__name__).version
    version = __version__
except DistributionNotFound:
    # package is not installed
    version = "Unknown"


def extension_metadata():
    import importlib.resources  # pylint: disable=import-error,no-name-in-module

    return {
        "priority": 10,
        "web_resources": {
            "static": [importlib.resources.files("WebAppDIRAC") / "WebApp" / "static"],  # pylint: disable=no-member
            "template": [importlib.resources.files("WebAppDIRAC") / "WebApp" / "template"],  # pylint: disable=no-member
        },
    }



""" Handler for Space Occupancy web App
"""

import json

from WebAppDIRAC.Lib.WebHandler import WebHandler, WErr, asyncGen
from DIRAC import gLogger
from DIRAC.ResourceStatusSystem.Client.ResourceManagementClient import ResourceManagementClient


class SpaceOccupancyHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getSelectionData(self):
        callback = {
            "StorageElement": set(),
        }

        rmc = ResourceManagementClient()

        gLogger.info("Arguments to web_getSelectionData", repr(self.request.arguments))

        spaces = yield self.threadTask(rmc.selectSpaceTokenOccupancyCache)

        if spaces["OK"]:
            for sp in spaces["Value"]:
                callback["StorageElement"].add(sp[1])

        for key, value in callback.items():
            callback[key] = [[item] for item in list(value)]
            # callback[key].sort()
            callback[key] = [["All"]] + callback[key]

        self.finish(callback)

    @asyncGen
    def web_getSpaceOccupancyData(self):

        rmc = ResourceManagementClient()

        se = json.loads(self.get_argument("StorageElement", "null"))

        res = yield self.threadTask(rmc.selectSpaceTokenOccupancyCache, None, list(se) if se else se)

        if not res["OK"]:
            raise WErr.fromSERROR(res)

        resList = []
        for sp in res["Value"]:
            # sp is e.g. ['dips://lbtestvobox.cern.ch:9196/',
            #             'CertificationSandboxSE',
            #             0.0,
            #             76085.6171875,
            #             161137.355469,
            #             datetime.datetime(2019, 7, 15, 11, 17, 38)]
            spRes = {}
            spRes["Endpoint"] = sp[0]
            spRes["StorageElement"] = sp[1]
            spRes["LastCheckTime"] = str(sp[5])

            if sp[4]:
                spRes["Ratio"] = float("%.2f" % (sp[3] * 100 / sp[4]))
            else:
                spRes["Ratio"] = "-"

            spRes["Free"] = float("%.2f" % sp[3])
            spRes["Total"] = float("%.2f" % sp[4])
            spRes["Guaranteed"] = float("%.2f" % sp[2])

            # FIXME: call here SpaceTokenOccupancyPolicy and avoid hardcoding twice

            if sp[4] == 0:
                spRes["Status"] = "Unknown"
            elif sp[3] < 0.1:
                spRes["Status"] = "Banned"
            elif sp[3] < 5:
                spRes["Status"] = "Degraded"
            else:
                spRes["Status"] = "Active"

            resList.append(spRes)

        self.finish({"success": "true", "result": resList, "total": len(res["Value"])})



import json

from DIRAC import gConfig, gLogger
from DIRAC.Core.Utilities import Time
from DIRAC.WorkloadManagementSystem.Client.PilotManagerClient import PilotManagerClient
from DIRAC.WorkloadManagementSystem.Client.JobMonitoringClient import JobMonitoringClient
from WebAppDIRAC.Lib.WebHandler import WebHandler, asyncGen


class PilotSummaryHandler(WebHandler):

    AUTH_PROPS = "authenticated"

    @asyncGen
    def web_getPilotSummaryData(self):
        callback = {}
        req = self.__request()

        result = yield self.threadTask(
            PilotManagerClient().getPilotSummaryWeb, req, self.globalSort, self.pageNumber, self.numberOfJobs
        )

        if not result["OK"]:
            self.finish({"success": "false", "result": [], "total": 0, "error": result["Message"]})
            return

        result = result["Value"]

        if "TotalRecords" not in result:
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (result["TotalRecords"] > 0):
            self.finish(
                {"success": "false", "result": [], "total": 0, "error": "There were no data matching your selection"}
            )
            return

        if not ("ParameterNames" in result and "Records" in result):
            self.finish({"success": "false", "result": [], "total": -1, "error": "Data structure is corrupted"})
            return

        if not (len(result["ParameterNames"]) > 0):
            self.finish({"success": "false", "result": [], "total": -1, "error": "ParameterNames field is missing"})
            return

        if not (len(result["Records"]) > 0):
            self.finish({"success": "false", "result": [], "total": 0, "Message": "There are no data to display"})
            return

        callback = []
        jobs = result["Records"]
        head = result["ParameterNames"]
        headLength = len(head)

        for i in jobs:
            tmp = {}
            for j in range(0, headLength):
                tmp[head[j]] = i[j]
            callback.append(tmp)
        total = result["TotalRecords"]
        total = result["TotalRecords"]
        timestamp = Time.dateTime().strftime("%Y-%m-%d %H:%M [UTC]")
        if "Extras" in result:
            st = self.__dict2string({})
            extra = result["Extras"]
            callback = {
                "success": "true",
                "result": callback,
                "total": total,
                "extra": extra,
                "request": st,
                "date": timestamp,
            }
        else:
            callback = {"success": "true", "result": callback, "total": total, "date": timestamp}
        self.finish(callback)

    def __dict2string(self, req):
        result = ""
        try:
            for key, value in req.iteritems():
                result = result + str(key) + ": " + ", ".join(value) + "; "
        except Exception as x:
            pass
            gLogger.info("\033[0;31m Exception: \033[0m %s" % x)
        result = result.strip()
        result = result[:-1]
        return result

    @asyncGen
    def web_getSelectionData(self):
        callback = {}
        group = self.getUserGroup()
        user = self.getUserName()
        if user == "Anonymous":
            self.finish({"success": "false", "result": [], "total": 0, "error": "Insufficient rights"})
        else:
            result = yield self.threadTask(JobMonitoringClient().getSites)
            if result["OK"]:
                tier1 = gConfig.getValue("/WebApp/PreferredSites")
                if tier1:
                    try:
                        tier1 = tier1.split(", ")
                    except BaseException:
                        tier1 = list()
                else:
                    tier1 = list()
                site = []
                if len(result["Value"]) > 0:
                    s = list(result["Value"])
                    for i in tier1:
                        site.append([str(i)])
                    for i in s:
                        if i not in tier1:
                            site.append([str(i)])
                else:
                    site = [["Nothing to display"]]

            else:
                site = [["Error during RPC call"]]

            callback["site"] = site
            callback["Status"] = [["Good"], ["Bad"], ["Idle"], ["Poor"], ["Fair"]]

            self.finish(callback)

    ################################################################################

    def __request(self):
        self.numberOfJobs = int(self.get_argument("limit", "25"))
        self.pageNumber = int(self.get_argument("start", "0"))
        self.globalSort = [["GridSite", "ASC"]]
        group = self.getUserGroup()
        user = self.getUserName()

        req = {}
        found = False
        jobids = list(json.loads(self.get_argument("id", "[]")))
        if jobids:
            req["JobID"] = jobids
            found = True

        elif "expand" in self.request.arguments:
            expand = list(json.loads(self.get_argument("expand", "[]")))
            if expand:
                self.numberOfJobs = 500
                self.pageNumber = 0
                req["ExpandSite"] = expand[0]
                found = True

        if not found:
            value = list(json.loads(self.get_argument("prod", "[]")))
            if value:
                req["JobGroup"] = value

            value = list(json.loads(self.get_argument("site", "[]")))
            if len(value) > 1:
                req["GridSite"] = value
            elif len(value) == 1:
                req["ExpandSite"] = value[0]

            value = list(json.loads(self.get_argument("Status", "[]")))
            if value:
                req["Status"] = value

            if "sort" in self.request.arguments:
                sort = json.loads(self.get_argument("sort"))
                if len(sort) > 0:
                    self.globalSort = [[i["property"], i["direction"]] for i in sort]
                else:
                    self.globalSort = [["GridSite", "DESC"]]

        if self.get_argument("startDate", ""):
            req["FromDate"] = self.get_argument("startDate")
            if self.get_argument("startTime", ""):
                req["FromDate"] += " " + self.get_argument("startTime")

        if self.get_argument("endDate", ""):
            req["ToDate"] = self.get_argument("endDate")
            if self.get_argument("endTime", ""):
                req["ToDate"] += " " + self.get_argument("endTime")

        if self.get_argument("date", ""):
            req["LastUpdate"] = self.get_argument("date")
        gLogger.info("REQUEST:", req)
        return req
